package org.springblade.pson.controller;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.Event;
import org.springblade.pson.service.IEventService;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;

/**
 * <p>
 * 事件表 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-06-07
 */
@RequestMapping("/event")
@RestController
@AllArgsConstructor
@Api(value = "事件", tags = "事件管理接口")
public class EventController extends BladeController {

	private final IEventService eventService;


	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增事件")
	public R save(@Valid @RequestBody Event event) {
		event.setCreateTime(new Date());
		return R.status(eventService.save(event));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除事件")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(eventService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改事件")
	public R update(@Valid @RequestBody Event event) {
		return R.status(eventService.updateById(event));
	}


	@GetMapping("/page")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "分页")
	public R<IPage<Event>> list(Event event, Query query,String eventName) {
		IPage<Event> pages = eventService.page(Condition.getPage(query), Condition.getQueryWrapper(event));
		if(StrUtil.isNotEmpty(eventName)){
			 pages = eventService.page(Condition.getPage(query), new QueryWrapper<Event>().like("name",eventName));
		}
		return R.data(pages);
	}

	@GetMapping("/list")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "列表")
	public R list(String platform) {
		List<Event> list = eventService.getListByPlatform(platform);
		return R.data(list);
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "详情")
	public R<Event> detail(Event event) {
		Event detail = eventService.getOne(Condition.getQueryWrapper(event));
		return R.data(detail);
	}


	@GetMapping("/select/list")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "事件下拉列表")
	public R selectList() {
		List<Event> list = eventService.list(
			new LambdaQueryWrapper<Event>()
				.isNotNull(Event::getId)
		);
		List<Map<String, Object>> data = list.stream()
			.map(obj -> {
				Map<String, Object> map = new HashMap<>();
				map.put("id", obj.getId());
				map.put("name", obj.getName());
				return map;
			}).collect(Collectors.toList());
		return R.data(data);
	}



}

