package org.springblade.pson.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.PsonEvent;
import org.springblade.pson.service.IPsonEventService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户画像-埋点事件表 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-04-16
 */
@RestController
@RequestMapping("Pson/psonEvent")
@AllArgsConstructor
@Api(value = "用户画像-埋点事件", tags = "用户画像-埋点事件管理接口")
public class PsonEventController extends BladeController {


	private final IPsonEventService psonEventService;


	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增埋点事件", notes = "传入psonEvent")
	public R save(@Valid @RequestBody PsonEvent psonEvent) {
		return R.status(psonEventService.save(psonEvent));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除埋点事件", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(psonEventService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改埋点事件", notes = "传入psonEvent")
	public R update(@Valid @RequestBody PsonEvent psonEvent) {
		return R.status(psonEventService.updateById(psonEvent));
	}


	@GetMapping("/list")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "列表", notes = "传入psonEvent")
	public R<IPage<PsonEvent>> list(PsonEvent psonEvent, Query query) {
		IPage<PsonEvent> pages = psonEventService.page(Condition.getPage(query), Condition.getQueryWrapper(psonEvent));
		return R.data(pages);
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "详情", notes = "传入psonEvent")
	public R<PsonEvent> detail(PsonEvent psonEvent) {
		PsonEvent detail = psonEventService.getOne(Condition.getQueryWrapper(psonEvent));
		return R.data(detail);
	}


	@GetMapping("/select/list")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "埋点事件下拉列表")
	public R selectList() {
		List<PsonEvent> list = psonEventService.list(
			new LambdaQueryWrapper<PsonEvent>()
				.isNotNull(PsonEvent::getId)
		);
		List<Map<String, Object>> data = new ArrayList<>();
		Map<String, Object> maps = new HashMap<>();
		maps.put("id", "0");
		maps.put("name", "通用");
		data.add(maps);
		for (PsonEvent psonEvent : list) {
			Map<String, Object> map = new HashMap<>();
			map.put("id", psonEvent.getId());
			map.put("name", psonEvent.getTagEventName());
			data.add(map);
		}
		return R.data(data);
	}

}

