package org.springblade.pson.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.PsonProject;
import org.springblade.pson.service.IPsonProjectService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户画像-埋点项目 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-04-16
 */
@RestController
@RequestMapping("Pson/psonProject")
@AllArgsConstructor
@Api(value = "用户画像-埋点项目", tags = "用户画像-埋点项目管理接口")
public class PsonProjectController extends BladeController {


	private final IPsonProjectService psonProjectService;


	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增埋点项目", notes = "传入psonProject")
	public R save(@Valid @RequestBody PsonProject psonProject) {
		return R.status(psonProjectService.save(psonProject));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除埋点项目", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(psonProjectService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改埋点项目", notes = "传入psonEvent")
	public R update(@Valid @RequestBody PsonProject psonProject) {
		return R.status(psonProjectService.updateById(psonProject));
	}


	@GetMapping("/list")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "列表", notes = "传入psonEvent")
	public R<IPage<PsonProject>> list(PsonProject psonProject, Query query) {
		IPage<PsonProject> pages = psonProjectService.page(Condition.getPage(query), Condition.getQueryWrapper(psonProject));
		return R.data(pages);
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "详情", notes = "传入psonEvent")
	public R<PsonProject> detail(PsonProject psonProject) {
		PsonProject detail = psonProjectService.getOne(Condition.getQueryWrapper(psonProject));
		return R.data(detail);
	}


	@GetMapping("/select/list")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "埋点项目下拉列表")
	public R selectList() {
		List<PsonProject> list = psonProjectService.list(
			new LambdaQueryWrapper<PsonProject>()
				.isNotNull(PsonProject::getId)
		);
		List<Map<String, Object>> data = new ArrayList<>();
		Map<String, Object> maps = new HashMap<>();
		maps.put("id", "0");
		maps.put("name", "通用");
		data.add(maps);
		for (PsonProject psonProject : list) {
			Map<String, Object> map = new HashMap<>();
			map.put("id", psonProject.getId());
			map.put("name", psonProject.getProjectName());
			data.add(map);
		}
		return R.data(data);
	}

}

