package org.springblade.pson.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.pson.entity.PsonProject;
import org.springblade.pson.entity.PsonUserAttr;
import org.springblade.pson.service.IPsonUserAttrService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * 用户画像-埋点用户属性 前端控制器
 * </p>
 *
 * @author zbc
 * @since 2021-04-16
 */
@RestController
@RequestMapping("Pson/psonUserAttr")
@AllArgsConstructor
@Api(value = "用户画像-埋点用户属性", tags = "用户画像-埋点用户属性管理接口")
public class PsonUserAttrController extends BladeController {


	private final IPsonUserAttrService psonUserAttrService;


	@PostMapping("/save")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "新增埋点用户属性", notes = "传入psonUserAttr")
	public R save(@Valid @RequestBody PsonUserAttr psonUserAttr) {
		return R.status(psonUserAttrService.save(psonUserAttr));
	}


	@PostMapping("/remove")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "删除埋点用户属性", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合  1,2,3", required = true) @RequestParam String ids) {
		return R.status(psonUserAttrService.removeByIds(Func.toLongList(ids)));
	}


	@PostMapping("/update")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "修改埋点用户属性", notes = "传入psonUserAttr")
	public R update(@Valid @RequestBody PsonUserAttr psonUserAttr) {
		return R.status(psonUserAttrService.updateById(psonUserAttr));
	}


	@GetMapping("/list")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "列表", notes = "传入psonUserAttr")
	public R<IPage<PsonUserAttr>> list(PsonUserAttr psonUserAttr, Query query) {
		IPage<PsonUserAttr> pages = psonUserAttrService.page(Condition.getPage(query), Condition.getQueryWrapper(psonUserAttr));
		return R.data(pages);
	}


	@GetMapping("/detail")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "详情", notes = "传入psonEvent")
	public R<PsonUserAttr> detail(PsonUserAttr psonUserAttr) {
		PsonUserAttr detail = psonUserAttrService.getOne(Condition.getQueryWrapper(psonUserAttr));
		return R.data(detail);
	}

}

