package org.springblade.sms.component;

import java.io.File;
import java.util.Random;

import org.springblade.core.tool.api.R;
import org.springframework.web.multipart.MultipartFile;

/**
 * 短信附件处理工具类
 * 
 * @author William
 * @date 2022年11月14日 下午4:31:16
 */
public class SmsAttachmentKit {

	private final static String imageSuffix = "jpg,png,gif"; // 图片允许的后缀
	private final static String videoSuffix = "mp4,3gp"; // 视频允许的后缀
	private final static String audioSuffix = "mp3,wav,midi"; // 音频允许的后缀

	private final static long maxSize = 1024 * 1024 * 18 / 10; // 文件大小限制,此处最大限制为1.8MB

	/**
	 * 保存短信附件
	 * 
	 * @param file
	 * @param uploadDir
	 * @return
	 */
	public static R<Object> save(MultipartFile file, String uploadDir) {
		String fileName = getMarking28() + '.' + getSuffixName(file.getOriginalFilename());

		File dirFile = new File(uploadDir);
		if (!dirFile.exists()) {
			dirFile.mkdirs();
		}

		try {
			long size = file.getSize();
			if (size > maxSize) {
				return R.fail("附件大小不可超过1.8MB");
			}

			boolean fileExtName = false;
			fileExtName |= checkSubffix(fileName, imageSuffix);
			fileExtName |= checkSubffix(fileName, videoSuffix);
			fileExtName |= checkSubffix(fileName, audioSuffix);

			if (!fileExtName) {
				return R.fail("不支持的文件格式，图片仅支持（jpg,png,gif），视频仅支持（mp4,3gp），音频仅支持（mp3,wav,midi）");
			}

			File outFile = new File(uploadDir + fileName);
			file.transferTo(outFile);
		} catch (Exception e) {
			return R.fail(e.getMessage());
		}

		return R.data(fileName);
	}

	/** 
	 * 验证指定文件名是否存在于指定后缀列表中 
	 * 参数：文件名、后缀列表	
	 * case：checkSubffix("123.jpg", "jpg,png,gif")	验证通过  
	 */
	static boolean checkSubffix(String fileName, String suffixList) {
		// 获取后缀，并转为小写
		String ext = getSuffixName(fileName).toLowerCase();
		// 去空格，加逗号
		String yxSuffix = suffixList.replace(" ", "") + ",";
		if (yxSuffix.indexOf(ext + ",") == -1) {
			return false;
		}

		return true;
	}

	/** 返回随机生成的唯一标示28位唯一标示符 */
	static String getMarking28() {
		return System.currentTimeMillis() + "" + new Random().nextInt(Integer.MAX_VALUE);
	}

	/** 取文件后缀 */
	static String getSuffixName(String fileName) {
		return fileName.substring(fileName.lastIndexOf(".") + 1).toLowerCase();
	}

}
