package org.springblade.sms.component;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springblade.core.tool.support.Kv;
import org.springblade.sms.bean.dto.AdsMarketSmsTemplate;
import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.bean.entity.TemplateAttr;
import org.springblade.sms.enums.GroupType;
import org.springblade.sms.enums.SmsTemplateSystemAttr;
import org.springblade.sms.enums.SmsType;
import org.springblade.sms.kafka.PushTaskProducer;
import org.springblade.sms.kafka.SmsDataKafka;
import org.springblade.sms.kafka.SmsStatusKafka;
import org.springblade.sms.service.AdsMarketSmsTemplateService;
import org.springblade.sms.service.AdsMarketTaskInfoLogService;
import org.springblade.sms.service.clickhouse.CubeGroupService;
import org.springblade.useranalysis.activespid.common.DateTimeKit;
import org.springblade.useranalysis.activespid.common.DateTimeType;
import org.springblade.useranalysis.activespid.entity.SpidSmsActiveLog;
import org.springblade.useranalysis.activespid.service.SpidSmsActiveLogService;
import org.springblade.useranalysis.common.utils.Tuple;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import cn.hutool.core.util.PhoneUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 短信发送服务
 *
 * @Author：william
 * @Date：2022/5/26
 */
@Slf4j
@Service
public class SmsService {
	public static final String CONTENT_ID_FOR_NORMAL = "2022032901";
	public static final String SMS_SIGN_FOR_NORMAL = "联通畅视";
	public static final int THREAD_POOL_SIZE = 100;

	@Autowired
	private AdsMarketSmsTemplateService templateService;

	@Autowired
	private AdsMarketTaskInfoLogService taskLogService;

	@Autowired
	private ShortLinkService shortLinkService;

	@Autowired
	private SpidSmsActiveLogService spidSmsLogService;

	@Autowired
	private SmsSender smsSender;

	@Autowired
	private CubeGroupService cubeGroupService;

	@Autowired
	private PushTaskProducer pushTaskProducer;

	private final ExecutorService threadPoolSms = Executors.newFixedThreadPool(THREAD_POOL_SIZE);
	private final ExecutorService threadPoolSpid = Executors.newFixedThreadPool(THREAD_POOL_SIZE);

	public void handleSmsSpidQueue(LinkedBlockingQueue<Tuple<String, String>> queue) {
		// windata测试后，打开该代码段
		if (!smsSender.isProd()) {
			return;
		}

		try {
			final String msg = "您有10GB芒果TV定向流量仍未激活使用，请在芒果TV客户端中登录激活。具体流程：芒果TV客户端“我的”-运营商专区 “我要免流”，点击页面下方“会员免流激活”，输入手机号验证码即可。详细操作可参考 https://live.mige.tv/platform/ytpxz/tutorial/mangotv.html";
			Tuple<String, String> tuple = queue.take();
			String mobile = tuple.getK();
			String spid = tuple.getV();
			String smsContent = PhoneUtil.hideBetween(mobile) + "，" + msg;
			Map<String, String> varMap = new HashMap<>();
			varMap.put("projects", SMS_SIGN_FOR_NORMAL);
			varMap.put("name", smsContent);
			threadPoolSpid.execute(() -> {
				sendSmsSpid(mobile, varMap, spid);
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 
	 * @param mobile
	 * @param varMap
	 * @param spid
	 */
	private void sendSmsSpid(String mobile, Map<String, String> varMap, String spid) {
		try {
			log.info("send spid sms,mobile is " + mobile);
			log.info("send spid sms,param is " + JSON.toJSONString(varMap));
			final String contentId = CONTENT_ID_FOR_NORMAL;
			SmsConfig smsConfig = SmsConfig.of(SmsType.notice);
			JSONObject smsResult = smsSender.sendSms(varMap, mobile, contentId, smsConfig);
			String smsBizId = "";
			if (smsResult != null) {
				String resultcode = smsResult.getString("resultcode");
				if (resultcode.equalsIgnoreCase("0")) {
					// 发送成功
					// 短信网关的任务ID
					smsBizId = smsResult.getString("taskid");
					// 芒果TV的促活短信暂不查询状态
//					SmsStatusKafka smsStatusData = new SmsStatusKafka();
//					smsStatusData.setMobile(mobile);
//					smsStatusData.setSmsBizId(smsBizId);
//					smsStatusData.setTimes(1);
//					smsStatusData.setSmsConfig(smsConfig);
//					pushTaskProducer.smsStatus(smsStatusData);
				}
			}
			Integer firstDay = DateTimeKit.getDateValue(DateTimeType.DAY, DateTimeKit.getPreDay(3));
			SpidSmsActiveLog data = new SpidSmsActiveLog();
			data.setActivityName("芒果合约包产品促活的需求开发");
			data.setApiName("");
			data.setBatchNo(String.valueOf(firstDay));
			data.setCategory("A");
			data.setSendTime(new Date());
			data.setSmsContent(varMap.get("name"));
			data.setSmsTaskId(smsBizId);
			data.setSpid(spid);
			data.setSpname("芒果19元产品（含合约）专用");
			data.setTaskPerson("杜伟");
			data.setUserId(mobile);
			spidSmsLogService.save(data);
		} catch (Exception e) {
			log.error("sendSmsSpid:" + e.getMessage(), e);
		}
	}

	public void handleSmsQueue(LinkedBlockingQueue<SmsDataKafka> queue) {
		// windata测试后，打开该代码段
		if (!smsSender.isProd()) {
			return;
		}

		try {
			SmsDataKafka smsData = queue.take();
			String mobile = smsData.getMobile();
			AdsMarketTaskInfo task = smsData.getTask();
			Integer templateId = task.getTplInfoId();
			AdsMarketSmsTemplate template = templateService.getById(templateId);
			String shortLink = shortLinkService.getShortLink(mobile, task);
			Map<String, String> varMap = buildSmsVars(mobile, task, template, shortLink);
			threadPoolSms.execute(() -> {
				sendSms(varMap, mobile, task);
			});
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 发送短信
	 * 
	 * 旧短信通用模板,短信内容格式及参数为：【{projects}】{name}
	 * 其中name变量对应的内容又为短信模板中对应的内容替换变量后的内容
	 * 
	 * 新申请的短信模板
	 * 格式一般为：【{projects}】亲爱的${user}用户你好，首月0.01元即可享受优酷月VIP卡+专属流量30G，点击${url}一键获取。
	 * 
	 * @param varMap 对应短信模板的参数和值
	 * @param mobile 手机号
	 * @param task 短信任务
	 */
	private void sendSms(Map<String, String> varMap, String mobile, AdsMarketTaskInfo task) {
		try {
			log.info("send sms,mobile is " + mobile);
			log.info("send sms,param is " + JSON.toJSONString(varMap));
			Integer templateId = task.getTplInfoId();
			AdsMarketSmsTemplate template = templateService.getById(templateId);
			String contentId = template.getContentId();
			SmsConfig smsConfig = SmsConfig.of(template.getSmsType());
			JSONObject smsResult = smsSender.sendSms(varMap, mobile, contentId, smsConfig);
			String smsBizId = "";
			if (smsResult != null) {
				String resultcode = smsResult.getString("resultcode");
				if (resultcode.equalsIgnoreCase("0")) {
					// 发送成功
					// 短信网关的任务ID
					smsBizId = smsResult.getString("taskid");
					SmsStatusKafka smsStatusData = new SmsStatusKafka();
					smsStatusData.setMobile(mobile);
					smsStatusData.setSmsBizId(smsBizId);
					smsStatusData.setTimes(1);
					smsStatusData.setSmsConfig(smsConfig);
					pushTaskProducer.smsStatus(smsStatusData);
				}
			}
			String url = varMap.containsKey("url") ? varMap.get("url") : "";
			String smsContent = getRealSmsContent(varMap, url, mobile, task, template);
			taskLogService.saveSmsLog(task, smsContent, mobile, smsContent, contentId, smsBizId);
		} catch (Exception e) {
			log.error("sendSms:" + e.getMessage(), e);
		}
	}

	/**
	 * 任务短信测试
	 *
	 * @param mobile 目标手机号
	 * @param task 测试短信对应的任务
	 * @param isSendSms 是否实际发送短信
	 * @return
	 */
	public Kv sendSmsForTest(String mobile, AdsMarketTaskInfo task, boolean isSendSms) {
		Kv data = Kv.create();
		data.set("phone", mobile);
		try {
			// 生成发送短信需要的参数
			AdsMarketSmsTemplate template = templateService.getById(task.getTplInfoId());
			SmsConfig smsConfig = SmsConfig.of(template.getSmsType());
			String shortLink = shortLinkService.getShortLink(mobile, task);
			Map<String, String> varMap = buildSmsVars(mobile, task, template, shortLink);
			data.set("smsContent", getRealSmsContent(varMap, shortLink, mobile, task, template));
			if (isSendSms) {
				JSONObject smsResult = smsSender.sendSms(varMap, mobile, template.getContentId(), smsConfig);
				if (smsResult != null) {
					data.set("smsResponse", smsResult);
				}
			}
		} catch (Exception e) {
			data.set("error", e);
		}
		return data;
	}

	private String getRealSmsContent(Map<String, String> varMap, String shortLink, String mobile,
			AdsMarketTaskInfo task, AdsMarketSmsTemplate template) {
		SmsType smsType = SmsType.reslove(task.getSmsType());
		String smsContent = "";
		if (smsType != SmsType.digital) {
			// 旧短信模板
			smsContent = varMap.get("name");
		} else {
			// 新的数字短信模板
			List<TemplateAttr> attrlist = intTemplateAttrList(task, mobile, shortLink);
			smsContent = buildSmsContent(template.getContent(), attrlist, task, mobile);
		}
		return smsContent;
	}

	private List<TemplateAttr> intTemplateAttrList(AdsMarketTaskInfo task, String phone, String url) {
		List<TemplateAttr> attrlist = JSON.parseArray(task.getAttrJson(), TemplateAttr.class);
		attrlist.add(new TemplateAttr("phone", phone));
		if (StrUtil.isNotBlank(url)) {
			attrlist.add(new TemplateAttr("url", url));
		}
		return attrlist;
	}

	/**
	 * 生成短信发送接口需要的vars参数
	 * 
	 * @param phone 手机号
	 * @param task 短信发送任务对象
	 * @param template 短信任务对应的短信模板
	 * @param url 短信中的链接
	 * @return
	 */
	private Map<String, String> buildSmsVars(String phone, AdsMarketTaskInfo task, AdsMarketSmsTemplate template,
			String url) {
		// 创建短信任务时前端传递的带值和默认值的变量属性列表
		List<TemplateAttr> attrlist = intTemplateAttrList(task, phone, url);

		SmsType smsType = SmsType.reslove(template.getSmsType());

		Map<String, String> varMap = new HashMap<String, String>();
		varMap.put("projects", template.getSmsSign());
		if (smsType == SmsType.digital) {
			// 新的数字短信接口
			for (TemplateAttr attr : attrlist) {
				String value = StrUtil.isBlank(attr.getAttrValue()) ? attr.getDefValue() : attr.getAttrValue();
				if (attr.getAttrCode().equalsIgnoreCase(SmsTemplateSystemAttr.phone.getCode())) {
					// 当前变量为手机号，需要判断是否加密
					if (task.getDesens() == 1) {
						// 手机号需要脱敏，隐藏中间四位
						value = PhoneUtil.hideBetween(value).toString();
					}
				}
				varMap.put(attr.getAttrCode(), value);
			}
		} else {
			// 新的短信接口，参数都用name传递
			String smsContent = buildSmsContent(template.getContent(), attrlist, task, phone);
			varMap.put("name", smsContent);
		}
		return varMap;
	}

	/**
	 * 生成短信的实际发送内容(不带短信签名)
	 *
	 * @param smsTemplateContent 短信模板的内容
	 * @param list  短信任务的变量值列表
	 * @param task  短信任务
	 * @param phone 手机号
	 * @return
	 */
	private String buildSmsContent(String smsTemplateContent, List<TemplateAttr> list, AdsMarketTaskInfo task,
			String phone) {
		// 如果变量包含手机号，则需要判断手机号是否隐藏
		Map<String, Object> params = new HashMap<>();
		for (TemplateAttr attr : list) {
			String value = StrUtil.isBlank(attr.getAttrValue()) ? attr.getDefValue() : attr.getAttrValue();
			if (attr.getAttrCode().equalsIgnoreCase(SmsTemplateSystemAttr.phone.getCode())) {
				// 当前变量为手机号，需要判断是否加密
				if (task.getDesens() == 1) {
					// 手机号需要脱敏，隐藏中间四位
					value = PhoneUtil.hideBetween(phone).toString();
				}
			}
			if (attr.getAttrType() != null && attr.getAttrType() == 2
					&& task.getGroupType() == GroupType.custom_group.getValue()) {
				// 动态字段并且是自定义人群，需要查询表数据来动态拼接
				value = cubeGroupService.getValueByColumn(task.getGroupId(), attr.getField(), attr.getDefValue(),
						phone);
			}
			params.put(attr.getAttrCode(), value);
		}
		String reg = "\\$\\{\\w*\\}";
		Pattern pattern = Pattern.compile(reg);
		Matcher matcher = pattern.matcher(smsTemplateContent);
		while (matcher.find()) {
			String group = matcher.group();
			String key = group.substring(2, group.length() - 1);
			if (!params.containsKey(key))
				throw new RuntimeException("未找到需要替换的key：" + key);
			smsTemplateContent = smsTemplateContent.replace(group, params.get(key).toString());
		}
		return smsTemplateContent;
	}
}
