package org.springblade.sms.controller;

import java.util.Date;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.support.Kv;
import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.bean.dto.AdsMarketTaskInfoChecker;
import org.springblade.sms.bean.params.TaskAddParam;
import org.springblade.sms.bean.params.TaskPageParam;
import org.springblade.sms.bean.vo.MarketTaskVo;
import org.springblade.sms.component.MarketTaskPushService;
import org.springblade.sms.component.SmsService;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.enums.SmsPushPattern;
import org.springblade.sms.enums.SmsPushStatus;
import org.springblade.sms.enums.TaskDraftStatus;
import org.springblade.sms.service.AdsMarketTaskInfoCheckerService;
import org.springblade.sms.service.AdsMarketTaskInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 短信任务接口
 * 
 * @author William
 * @date 2022年11月10日 上午11:11:10
 */
@RestController
@RequestMapping("/adsMarketTaskInfo")
@Api(value = "人群短信任务", tags = "人群短信任务")
public class AdsMarketTaskInfoController extends BladeController {
	/**
	 * 服务对象
	 */
	@Autowired
	private AdsMarketTaskInfoService adsMarketTaskInfoService;

	@Autowired
	private MarketTaskPushService adsMarketPushService;

	@Autowired
	private SmsService smsService;

	@Autowired
	private AdsMarketTaskInfoCheckerService checkerService;

	@GetMapping("/test/sms")
	@ApiOperation(value = "测试发短信")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "taskId", value = "任务ID", dataType = "string", required = true),
			@ApiImplicitParam(name = "mobile", value = "测试手机号", dataType = "string", required = true),
			@ApiImplicitParam(name = "isSendSms", value = "是否发送短信(true-发送；false-不发送)", dataType = "boolean", required = true),
	})
	public R<Kv> test(String mobile, boolean isSendSms, Long taskId) {
		AdsMarketTaskInfo task = adsMarketTaskInfoService.getById(taskId);
		if (task == null) {
			return R.fail("任务不存在");
		}
		if (StrUtil.isBlank(mobile)) {
			return R.fail("手机号不能为空");
		}

		return R.data(smsService.sendSmsForTest(mobile, task, isSendSms));
	}

	@GetMapping("/sortNo/list")
	@ApiOperation(value = "批号列表")
	public R<List<String>> getSortNoList() {
		return R.data(adsMarketTaskInfoService.getSortNoList());
	}

	@GetMapping("/checkSelf")
	@ApiOperation(value = "任务自检测")
	public R<List<String>> checkSelf(Long taskId) {
		AdsMarketTaskInfo task = adsMarketTaskInfoService.getById(taskId);
		if (task == null) {
			return R.fail("任务不存在");
		}
		String self = String.valueOf(SecureUtil.getUserName());
		if (!self.equalsIgnoreCase(task.getCreater())) {
			return R.fail("只有创建人才可以自检测");
		}
		task.setDraftStaus(TaskDraftStatus.not_draft.getValue());
		task.setCheckStatus(CheckStatus.waiting.getValue());
		task.setPushStatus(SmsPushStatus.waiting.getValue());
		return R.status(adsMarketTaskInfoService.updateById(task));
	}

	@GetMapping("/check")
	@ApiOperation(value = "任务审核")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "taskId", value = "任务ID", dataType = "string", required = true),
			@ApiImplicitParam(name = "status", value = "审核状态(1-通过；2-作废)", dataType = "integer", required = true),
	})
	public R<String> check(Long taskId, Integer status) {
		AdsMarketTaskInfo task = adsMarketTaskInfoService.getById(taskId);
		String checkerId = String.valueOf(SecureUtil.getUserId());
		if (task == null) {
			return R.fail("任务不存在");
		}
		if (task.getCheckStatus() != CheckStatus.waiting.getValue()) {
			return R.fail("该任务已经审核完成，不能重复审核");
		}
		if (!checkerService.exist(task.getTaskId(), checkerId)) {
			return R.fail("抱歉，您没有审核权限");
		}

		if (checkerService.isChecked(task.getTaskId(), checkerId)) {
			return R.fail("您已审核过，不能重复审审核");
		}

		AdsMarketTaskInfoChecker checker = checkerService.getChecker(taskId, checkerId);
		if (status == 1) {
			checker.setStatus(CheckStatus.ok.getValue());
		} else {
			checker.setStatus(CheckStatus.no.getValue());
			task.setCheckStatus(CheckStatus.no.getValue());
			task.setCheckTime(new Date());
			adsMarketTaskInfoService.updateById(task);
		}
		checker.setCheckTime(new Date());
		checkerService.updateById(checker);

		// 所有审核人员是否都审核完成
		boolean isAllChecked = checkerService.isAllChecked(taskId);
		// 是否全票通过
		boolean isCheckOk = checkerService.isCheckOk(taskId);
		if (isAllChecked && isCheckOk) {
			// 全票通过，更新任务审核状态
			task.setCheckTime(new Date());
			task.setCheckStatus(CheckStatus.ok.getValue());
			adsMarketTaskInfoService.updateById(task);

			SmsPushPattern pushPattern = SmsPushPattern.reslove(task.getPushPattern());
			if (pushPattern == SmsPushPattern.inTime) {
				// 及时推送
				adsMarketPushService.asyncPushSms(task, 1);
			} else if (pushPattern == SmsPushPattern.atTime) {
				// 定时推送
				// 此处不做处理，交给定时任务AtTimeSendJob自动处理
			} else if (pushPattern == SmsPushPattern.cycleTime) {
				// 周期性推送
				// 此处不做处理，交给定时任务CycleSendJob自动处理
			}
		}
		return R.success("操作成功");
	}

	/**
	 * 分页查询所有数据
	 *
	 * @param params 查询参数
	 * @return 所有数据
	 */
	@SuppressWarnings("unchecked")
	@PostMapping("/page")
	@ApiOperation(value = "分页")
	public R<Kv> page(@RequestBody TaskPageParam params) {
		StringBuffer where = new StringBuffer(" where 1=1 ");
		if (!StrUtil.isEmpty(params.getMissionName())) {
			where.append(" and t.mission_name like '%" + params.getMissionName() + "%'");
		}
		if (!StrUtil.isEmpty(params.getGroupName())) {
			where.append(" and t.group_name like '%" + params.getGroupName() + "%'");
		}
		if (!StrUtil.isEmpty(params.getSortNo())) {
			where.append(" and t.sort_no = '" + params.getSortNo() + "'");
		}
		if (!StrUtil.isEmpty(params.getCreater())) {
			where.append(" and t.creater = '" + params.getCreater() + "'");
		}
		if (params.getCheckStatus() != null) {
			where.append(" and t.check_status = " + params.getCheckStatus());
		}
		if (params.getTaskType() != null) {
			where.append(" and t.task_type = " + params.getTaskType());
		}
		if (params.getCheckTimeStart() != null) {
			where.append(" and t.check_time >= '" + DateUtil.formatDateTime(params.getCheckTimeStart()) + "'");
		}
		if (params.getCheckTimeEnd() != null) {
			where.append(" and t.check_time <= '" + DateUtil.formatDateTime(params.getCheckTimeEnd()) + "'");
		}

		StringBuffer join = new StringBuffer();
		if (StrUtil.isNotBlank(params.getCheckUserName())) {
			join.append(" left join ads_market_task_info_checker c on ");
			join.append(" t.task_id = c.task_id ");
			where.append(" and c.check_user_name like '%" + params.getCheckUserName() + "%'");
		}
		Kv data = adsMarketTaskInfoService.getPageList(join, where, params.getCurrent(), params.getSize());
		if (data.containsKey("list")) {
			List<MarketTaskVo> list = (List<MarketTaskVo>) data.get("list");
			if (CollUtil.isNotEmpty(list)) {
				list.forEach(task -> {
					task.setCheckers(checkerService.getTaskCheckerList(task.getTaskId()));
				});
			}
		}
		return R.data(data);
	}

	/**
	 * 通过主键查询单条数据
	 *
	 * @param id 主键
	 * @return 单条数据
	 */
	@GetMapping("/detail/{id}")
	@ApiOperation(value = "详情")
	public R<Kv> info(@PathVariable Long id) {
		Kv data = Kv.create();
		data.set("task", adsMarketTaskInfoService.getById(id));
		data.set("checkers", checkerService.getTaskCheckerList(id));
		return R.data(data);
	}

	/**
	 * 新增数据
	 *
	 * @param data 实体对象
	 * @return 新增结果
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增")
	public R<Object> save(@RequestBody TaskAddParam data) {
		return adsMarketTaskInfoService.insertOrUpdate(data);
	}

	/**
	 * 修改数据 只有草稿状态才能修改
	 *
	 * @param data 实体对象
	 * @return 修改结果
	 */
	@PutMapping("/update")
	@ApiOperation(value = "修改")
	public R<Object> update(@RequestBody TaskAddParam data) {
		return adsMarketTaskInfoService.insertOrUpdate(data);
	}

	/**
	 * 删除数据
	 *
	 * @param id 任务ID
	 * @return 删除结果
	 */
	@GetMapping("/remove")
	@ApiOperation(value = "删除")
	public R<String> remove(@RequestParam("id") Long id) {
		return R.status(this.adsMarketTaskInfoService.removeById(id));
	}

	/**
	 * 删除数据
	 *
	 * @param idList 主键结合
	 * @return 删除结果
	 */
	@PostMapping("/remove/batch")
	@ApiOperation(value = "删除（批量）")
	public R<String> removeBatch(@RequestParam("idList") List<Long> idList) {
		return R.status(this.adsMarketTaskInfoService.removeByIds(idList));
	}

}
