package org.springblade.sms.job;

import java.util.Date;
import java.util.List;

import org.springblade.sms.bean.dto.AdsMarketTaskInfo;
import org.springblade.sms.component.MarketTaskPushService;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.enums.SmsPushPattern;
import org.springblade.sms.enums.SmsPushStatus;
import org.springblade.sms.enums.TaskDraftStatus;
import org.springblade.sms.service.AdsMarketTaskInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import lombok.extern.slf4j.Slf4j;

/**
 * 短信定时发送定时检查任务
 * 
 * @author William
 * @date 2022年11月15日 下午5:07:26
 */
@Slf4j
@Configuration
@EnableScheduling
public class AtTimeSendJob {
	@Autowired
	private AdsMarketTaskInfoService taskService;

	@Autowired
	private MarketTaskPushService pushService;

	/**
	 * 1分钟检查一次
	 */
	@Scheduled(cron = "0 0/1 * * * ?")
	private void findAtTimeSmsTask() {
		try {
			Date now = new Date();
			// 查询是否有需要推送的任务
			LambdaQueryWrapper<AdsMarketTaskInfo> wrapper = Wrappers.lambdaQuery(AdsMarketTaskInfo.class)
					.eq(AdsMarketTaskInfo::getPushPattern, SmsPushPattern.atTime.getValue())
					.eq(AdsMarketTaskInfo::getPushStatus, SmsPushStatus.waiting.getValue())
					.eq(AdsMarketTaskInfo::getCheckStatus, CheckStatus.ok.getValue())
					.eq(AdsMarketTaskInfo::getDraftStaus, TaskDraftStatus.not_draft.getValue())
					.le(AdsMarketTaskInfo::getPushTime, now);
			List<AdsMarketTaskInfo> taskList = taskService.list(wrapper);
			// 更新推送状态为进行中【此处不使用批量更新，是考虑如果某个任务更新状态失败，无法识别】
			if (taskList != null && !taskList.isEmpty()) {
				log.info("AtTimeSendJob {} tasks", taskList.size());
				taskList.forEach(task -> {
					AdsMarketTaskInfo entity = new AdsMarketTaskInfo();
					entity.setTaskId(task.getTaskId());
					entity.setPushStatus(SmsPushStatus.doing.getValue());
					boolean ret = taskService.updateById(entity);
					if (ret) { // 更新成功，添加到推送队列
						pushService.asyncPushSms(task, 1);
					}
				});
			}
		} catch (Exception e) {
			log.info("AtTimeSendJob::findAtTimeSmsTask exception", e.getMessage());
		}
	}

}
