package org.springblade.sms.job;

import java.util.Date;
import java.util.List;

import org.springblade.sms.bean.dto.AdsMarketSmsTemplate;
import org.springblade.sms.bean.params.TemplateCheckResponse;
import org.springblade.sms.bean.params.TemplateCheckResponse.CheckData;
import org.springblade.sms.component.SmsConfig;
import org.springblade.sms.component.SmsSender;
import org.springblade.sms.enums.CheckStatus;
import org.springblade.sms.enums.SmsType;
import org.springblade.sms.service.AdsMarketSmsTemplateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import lombok.extern.slf4j.Slf4j;

/**
 * 短信模板审核状态查询任务
 * 
 * @author William
 * @date 2022年11月15日 下午5:12:43
 */
@Slf4j
@Configuration
@EnableScheduling
public class SmsTemplateCheckJob {

	@Autowired
	private AdsMarketSmsTemplateService templateService;

	@Autowired
	private SmsSender smsSender;

	/**
	 * 2分钟检查一次
	 */
	@Scheduled(cron = "0 0/3 * * * ?")
	public void checkStatusLoop() {
		try {
			LambdaQueryWrapper<AdsMarketSmsTemplate> wrapper = Wrappers.lambdaQuery(AdsMarketSmsTemplate.class)
					.eq(AdsMarketSmsTemplate::getCheckStatus, CheckStatus.waiting.getValue())
					.eq(AdsMarketSmsTemplate::getSmsType, SmsType.digital.getValue());
			List<AdsMarketSmsTemplate> templateList = templateService.list(wrapper);
			if (templateList != null && !templateList.isEmpty()) {
				for (AdsMarketSmsTemplate template : templateList) {
					TemplateCheckResponse tcr = smsSender.qrytempstatus(template.getContentId(),
							SmsConfig.of(template.getSmsType()));
					if (tcr.isSuccess()) {
						// 接口调用成功
						// 0:通过；1:待处理；2:审核中；3:审核未通过；4:上传失败
						CheckData checkData = tcr.getData();
						if (checkData.getStatus().equalsIgnoreCase("0")) {
							// 申请成功
							template.setCheckTime(new Date());
							template.setCheckStatus(CheckStatus.ok.getValue());
							template.setCheckRemark("申请通过");
							templateService.saveOrUpdate(template);
						} else if (checkData.getStatus().equalsIgnoreCase("3")
								|| checkData.getStatus().equalsIgnoreCase("4")) {
							// 申请失败
							template.setCheckTime(new Date());
							template.setCheckStatus(CheckStatus.no.getValue());
							template.setCheckRemark(checkData.getResultinfo());
							templateService.saveOrUpdate(template);
						}
					} else {
						// 接口调用失败，有可能是ip没在白名单
					}
				}
			}
		} catch (Exception e) {
			log.info("SmsTemplateCheckJob::checkStatusLoop exception", e.getMessage());
		}
	}

}
