package org.springblade.sms.service.clickhouse;

import java.io.Serializable;
import java.util.List;

import org.springblade.sms.bean.dto.DefineGroup;
import org.springblade.sms.mapper.clickhouse.CubeGroupMapper;
import org.springblade.sms.service.DefineGroupService;
import org.springblade.useranalysis.common.utils.JsonMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.xiaoymin.knife4j.core.util.StrUtil;

import cn.hutool.core.collection.CollUtil;

/**
 * 根据自定义人群ID在clickhouse中查询手机号
 * 
 * @author William
 * @date 2022年11月14日 上午11:33:09
 */
@Service
@DS("clickhouseCubeGroup")
public class CubeGroupService extends ServiceImpl<CubeGroupMapper, JSONObject> {

	@Autowired
	private DefineGroupService groupService;

	public List<String> getPhoneList(DefineGroup group, int offset, int pageSize) {
		if (group == null) {
			return null;
		}
		String columnJson = group.getResultColumns();
		List<JsonMap> columnList = JsonMap.parseArray(columnJson, JsonMap.class);
		String phoneColumn = columnList.get(0).getString("columnName");
		String tableName = group.getResultTable();
		String timeZone = group.getTimeZone();
		String sqlLimit = "limit " + offset + "," + pageSize;
		return baseMapper.getPhoneList(phoneColumn, tableName, timeZone, sqlLimit);
	}

//	public long getPhoneTotal(DefineGroup group) {
//		if (group == null) {
//			return 0;
//		}
//		String columnJson = group.getResultColumns();
//		List<JsonMap> columnList = JsonMap.parseArray(columnJson, JsonMap.class);
//		String phoneColumn = columnList.get(0).getString("columnName");
//		String tableName = group.getResultTable();
//		String timeZone = group.getTimeZone();
//		return baseMapper.getPhoneTotal(phoneColumn, tableName, timeZone);
//	}

	/**
	 * 根据人群id和动态字段名获取值
	 * 
	 * @param groupId 自定义人群ID
	 * @param column 字段名
	 * @param defValue 默认值
	 * @param phone 手机号作为查询条件
	 * @return
	 */
	public String getValueByColumn(Serializable groupId, String column, String defValue, String phone) {
		DefineGroup group = groupService.getById(groupId);
		if (group == null) {
			return null;
		}
		String columnJson = group.getResultColumns();
		List<JsonMap> columnList = JsonMap.parseArray(columnJson, JsonMap.class);
		String phoneColumn = columnList.get(0).getString("columnName");
		String tableName = group.getResultTable();
		String timeZone = group.getTimeZone();
		List<JSONObject> list = baseMapper.getValueByColumn(tableName, timeZone, phoneColumn, phone);
		String value = null;

		if (CollUtil.isNotEmpty(list)) {
			JSONObject jo = list.get(0);
			if (jo.containsKey(column)) {
				Object obj = jo.get(column);
				if (obj != null) {
					value = obj.toString();
				}
			}
		}
		return StrUtil.isNotBlank(value) ? value : defValue;
	}
}
