package org.springblade.useranalysis.activespid.common;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.NumberUtil;

/**
 * Desc: 日期时间工具类
 * <p>
 * Modify:
 * <p>
 * Author: William Lynn
 * <p>
 * Create Date：Sep 11, 2021 4:31:03 PM
 * 
 */
public class DateTimeKit {

	final static String monthPattern = "yyyyMM";
	final static String dayPattern = "yyyyMMdd";
	final static String hourPattern = "yyyyMMddHH";

	public static void main(String[] args) {
		Integer month = DateTimeKit.getDateValue(DateTimeType.MONTH, DateTimeKit.getPreDay(3));
		Integer firstDay = DateTimeKit.getDateValue(DateTimeType.DAY, DateTimeKit.getPreDay(3));
		Integer secondData = DateTimeKit.getDateValue(DateTimeType.DAY, DateTimeKit.getPreDay(2));
		Integer thirdDay = DateTimeKit.getDateValue(DateTimeType.DAY, DateTimeKit.getPreDay(1));

		System.out.println(month);
		System.out.println(firstDay);
		System.out.println(secondData);
		System.out.println(thirdDay);
	}

	public Date ofDateTime(LocalDateTime localDateTime) {
		Date date = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
		return date;
	}

	public static String of(LocalDateTime localDateTime) {
		return LocalDateTimeUtil.format(localDateTime, DatePattern.NORM_DATETIME_FORMATTER);
	}

	/**
	 * 格式化日期为整数
	 * 
	 * @param type
	 * @param dataTime
	 * @return
	 */
	public static int getDateValue(DateTimeType type, LocalDateTime dataTime) {
		String dateValue = "0";
		if (type == DateTimeType.MONTH) {
			dateValue = LocalDateTimeUtil.format(dataTime, monthPattern);
		} else if (type == DateTimeType.DAY) {
			dateValue = LocalDateTimeUtil.format(dataTime, dayPattern);
		} else if (type == DateTimeType.HOUR) {
			dateValue = LocalDateTimeUtil.format(dataTime, hourPattern);
		}

		int value = NumberUtil.parseInt(dateValue);
		return value;
	}

	/**
	 * 整数日期类型转换为日期时间类型
	 * 
	 * @param type
	 * @param dateValue
	 * @return
	 */
	public static LocalDateTime getDateTime(DateTimeType type, int dateValue) {
		if (type == DateTimeType.MONTH) {
			// 月维度
			return LocalDateTimeUtil.parse(String.valueOf(dateValue), monthPattern);
		} else if (type == DateTimeType.DAY) {
			// 日维度
			return LocalDateTimeUtil.parse(String.valueOf(dateValue), dayPattern);
		} else if (type == DateTimeType.HOUR) {
			// 小时维度
			return LocalDateTimeUtil.parse(String.valueOf(dateValue), hourPattern);
		}
		return null;
	}

	/**
	 * 计算从当前时间往前几天的日期
	 * 
	 * @param days
	 * @return
	 */
	public static LocalDateTime getPreDay(long days) {
		LocalDateTime now = LocalDateTime.now();
		LocalDateTime nowDay = LocalDateTimeUtil.beginOfDay(now);
		return nowDay.minusDays(days);
	}
}
