package org.springblade.useranalysis.common.utils;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.function.BiConsumer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

/**
 * 
 * @author Administrator
 *
 */
public class ConvertKit {

	final static Logger LOGGER = LoggerFactory.getLogger(ConvertKit.class);
	
	/**
	 * 单个对象转换
	 * @param <T>
	 * @param <K>
	 * @param object
	 * @param clz
	 * @return
	 */
	public static <T, K extends Serializable> T transfer(K object, Class<T> clz, BiConsumer<T, K> function, String... ignoreProperties) {
		if(object == null) {
			return null;
		}
		try {
			T value = clz.newInstance();
			BeanUtils.copyProperties(object, value, ignoreProperties);
			if(function != null) {
				function.accept(value, object);
			}
			return value;
		} catch (Exception e) {
			LOGGER.error(e.getMessage(), e);
		}
		return null;
	}
	
	/**
	 * 对象列表转换
	 * @param <T>
	 * @param <K>
	 * @param objectList
	 * @param clz
	 * @return
	 */
	public static <T, K extends Serializable> List<T> transfer(List<K> objectList, Class<T> clz, BiConsumer<T, K> function, String... ignoreProperties){
		if(CollectionUtils.isEmpty(objectList)) {
			return new ArrayList<T>();
		}
		List<T> resultList = new ArrayList<T>();
		for(K obj : objectList) {
			try {
				resultList.add(transfer(obj, clz, function, ignoreProperties));
			} catch (Exception e) {
				LOGGER.error(e.getMessage(), e);
			}
		}
		return resultList;
	}
	
	/**
	 * 列表数据转换
	 * @param <T>
	 * @param <K>
	 * @param objectList
	 * @param clz
	 * @param ignoreProperties
	 * @return
	 */
	public static <T, K extends Serializable> List<T> transfer(List<K> objectList, Class<T> clz, String... ignoreProperties){
		return transfer(objectList, clz, null, ignoreProperties);
	}
	
	/**
	 * bean转换
	 * @param <T>
	 * @param <K>
	 * @param object
	 * @param clz
	 * @param ignoreProperties
	 * @return
	 */
	public static <T, K extends Serializable> T transfer(K object, Class<T> clz, String... ignoreProperties) {
		return transfer(object, clz, null, ignoreProperties);
	}
	
}
