package org.springblade.useranalysis.controller;

import java.util.List;
import java.util.Map;

import org.springblade.pson.entity.Event;
import org.springblade.pson.service.IEventService;
import org.springblade.useranalysis.common.utils.Constants;
import org.springblade.useranalysis.dto.ResultSegment;
import org.springblade.useranalysis.service.EventListService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;

/**
 * 事件分析模型
 */
@RestController
@RequestMapping("/v1/DataAnalysis")
@AllArgsConstructor
@Api(value = "用户行为分析", tags = "用户行为分析")
public class AnalysisModeController {

    @Autowired
    private IEventService eventService;

    @Autowired
    private EventListService eventListService;



    /**
     * 事件选项下拉菜单
     */
    @ApiOperation(value = "事件选项下拉菜单")
    @RequestMapping(value = "/eventList", method = RequestMethod.POST)
    public ResultSegment getEventList() {
        List<Map<String, Object>> maps = eventService.listMaps(new QueryWrapper<Event>().select("name", "code"));
//        List<Map<String, Object>> eventLists = eventListService.eventList();
        return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, maps);
    }


    /**
     * 细分维度[设备属性/用户属性] 下拉菜单
     */
    // 细分维度[设备属性/用户属性] 下拉菜单
    @ApiOperation("细分维度[设备属性/用户属性] 下拉菜单")
    @RequestMapping(value = "/attributeList", method = RequestMethod.POST)
    public ResultSegment getAttributeList() {
        List<Map<String, Object>> attributeList = eventListService.attributeList();
        return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, attributeList);
    }

    /**
     * 下拉菜单过滤条件Key
     */
    @ApiOperation(value = "下拉菜单过滤条件Key")
    @RequestMapping(value = "/dropDownListKey", method = RequestMethod.POST)
    public ResultSegment dropDownListKey() {
        JSONObject map = eventListService.dropDownListKey();
        return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
    }

    /**
     * 下拉菜单过滤条件Value
     */
    @ApiOperation(value = "下拉菜单过滤条件Value")
    @RequestMapping(value = "/dropDownListValue", method = RequestMethod.POST)
    public ResultSegment dropDownListValue(@RequestParam(value = "level_2_name", required = false) String level_2_name) {
        JSONObject map = eventListService.dropDownListValue(level_2_name);
        return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
    }

    /**
     * 事件选项下拉菜单
     */

    @ApiOperation(value = "事件选项下拉菜单")
    @RequestMapping(value = "/dropDownList", method = RequestMethod.POST)
    public ResultSegment dropDownList() {
        JSONObject map = eventListService.dropDownList();
        return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
    }

}
