package org.springblade.useranalysis.controller;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.useranalysis.common.utils.Constants;
import org.springblade.useranalysis.common.utils.UserExcelImportListener;
import org.springblade.useranalysis.dto.ResultSegment;
import org.springblade.useranalysis.dto.param.ExcelImportPageParam;
import org.springblade.useranalysis.entity.UserExcelImportLog;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.clickhouse.UserExcelImportPhone;
import org.springblade.useranalysis.enums.UserGroupProcessStatus;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.normal.UserExcelImportLogService;
import org.springblade.useranalysis.service.normal.UserExcelImportPhoneService;
import org.springblade.useranalysis.service.normal.UserGroupSumDataManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.LineHandler;
import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.StrUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

/**
 * @author william
 *
 */
@Slf4j
@RestController
@RequestMapping("/v1/phone/excel")
@Api(value = "导入人群手机号", tags = "导入人群手机号")
public class UserExcelImportController extends BladeController {

	@Autowired
	private UserExcelImportLogService importLogService;

	@Autowired
	private UserExcelImportPhoneService importPhoneService;
	
	@Autowired
	private UserGroupService userGroupService;
	
	@Autowired
	private UserGroupSumDataManager userGroupSumDataManager;

	@ApiOperation("导入Excel")
	@RequestMapping(value = "/import", method = RequestMethod.POST)
	public R<String> imports(
			@RequestParam String name, 
			//是否创建人群（1-是，0-否）
			@RequestParam(required = false, defaultValue = "0") Integer userGroupStatus, 
			@RequestParam MultipartFile file,
			//添加人群分析时间
			String startAnalyseDate, String endAnalyseDate
			) {
		if (StrUtil.isBlank(name) || file == null) {
			return R.fail("缺少参数");
		}

//		String fileName = file.getOriginalFilename();
//		if (!(fileName.endsWith(".xls") || fileName.endsWith(".xlsx"))) {
//			return R.fail("文件格式错误");
//		}

		String sysUserName = "管理员";
		Long sysUserId = 0l;

		BladeUser user = getUser();
		if (user != null) {
			sysUserName = getUser().getNickName();
			sysUserId = getUser().getUserId();
		}

		UserExcelImportLog excel = new UserExcelImportLog();
		Long id = IdWorker.getId();
		excel.setId(id);
		excel.setName(name);
		excel.setSysUserId(sysUserId);
		excel.setSysUserName(sysUserName);
		excel.setUserGroupStatus(userGroupStatus);
		excel.setCreateTime(new Date());

		boolean flag = false;
		if (importLogService.save(excel)) {
			flag = true;
			try {
				long count = 0;
				//支持txt文件的手机号
				if(file.getOriginalFilename().endsWith(".txt")) {
					long[] index = {0L};
					List<UserExcelImportPhone> lineList = new ArrayList<>();
					IoUtil.readUtf8Lines(file.getInputStream(), new LineHandler() {
						@Override
						public void handle(String line) {
							if(PhoneUtil.isMobile(line)){//过滤非手机号
								UserExcelImportPhone data = new UserExcelImportPhone();
								data.setPhone(line);
								data.setId(String.valueOf(id));
								lineList.add(data);
								index[0]++;
							}
						}
					});
					count = index[0];
					importPhoneService.saveBatch(lineList);
				}else {
					// 解析excel，并且插入clickhouse
					UserExcelImportListener listener = new UserExcelImportListener(id, importPhoneService);
					InputStream inputStream = file.getInputStream();
					EasyExcel.read(inputStream, UserExcelImportPhone.class, listener).sheet().headRowNumber(-1).doRead();
					count = listener.getTotal();
				}
				excel.setCount(count);
				boolean ret = importLogService.saveOrUpdate(excel);
				if(ret && userGroupStatus == 1) { //是否需要创建人群：是，创建个人群
					UserGroupEntity userGroupEntity = new UserGroupEntity();
					userGroupEntity.setReachCount("0");
					userGroupEntity.setGroupName(name + "[导入人群自动创建]");
					userGroupEntity.setImportId(excel.getId().toString());
					userGroupEntity.setRuleVersion(1); //设置为新版本rule规则
					userGroupEntity.setCreater(getUser().getNickName());
					userGroupEntity.setProcessStatus(UserGroupProcessStatus.waiting);
					userGroupEntity.setCreateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
					if(startAnalyseDate != null && endAnalyseDate != null) {
						List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.parse(startAnalyseDate, "yyyy-MM"), DateUtil.parse(endAnalyseDate, "yyyy-MM"), DateField.MONTH);
						List<String> analyseDateList = rangeToList.stream().map(item -> DateUtil.format(item, "yyyyMM")).sorted().collect(Collectors.toList());
						userGroupEntity.setAnalyseDateJson(JSON.toJSONString(analyseDateList));
					}
					if (userGroupService.save(userGroupEntity)) {
						//开始数据预处理
						userGroupSumDataManager.execute(userGroupEntity.getGroupId());
						log.info("添加人群成功，数据预处理已进入队列...");
					} else {
						log.warn("添加人群失败,importId=" + excel.getId().toString());
					}
				}
				log.info("UserExcelImport success,the count is  " + count);
			} catch (Exception e) {
				log.info("UserExcelImport fail,the error is: " + e.getMessage());
				importLogService.removeById(id);
				return R.fail("导入失败");
			}
		}
		return flag ? R.data(String.valueOf(id), "导入成功") : R.fail("导入失败");

	}

	@ApiOperation("分页查询")
	@RequestMapping(value = "/page", method = RequestMethod.POST)
	public ResultSegment page(@RequestBody ExcelImportPageParam params) {
		Long sysUserId = getUser().getUserId();
		if (sysUserId == null) {
			return new ResultSegment(false, 400, "没有权限");
		}

		QueryWrapper<UserExcelImportLog> wrapper = new QueryWrapper<>();
		wrapper.orderByDesc("id");
		wrapper.like(StrUtil.isNotEmpty(params.getName()), "name", params.getName());
		wrapper.like(StrUtil.isNotEmpty(params.getSysUserName()), "sys_user_name", params.getSysUserName());
		Page<UserExcelImportLog> logPage = new Page<>(params.getPageNo(), params.getPageSize());
		Page<UserExcelImportLog> excelLogPage = importLogService.page(logPage, wrapper);
		ResultSegment resultSegment = new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG,
				excelLogPage);
		return resultSegment;
	}

	@ApiOperation("删除导入的人群")
	@GetMapping("/delete/{id}")
	public R<String> delete(@PathVariable(value = "id", required = true) Long id) throws IOException {
		Long sysUserId = getUser().getUserId();
		if (sysUserId == null) {
			return R.fail("没有权限");
		}

		UserExcelImportLog excel = importLogService.getById(id);
		if (excel == null) {
			return R.fail("人群不存在");
		}

		if (sysUserId.longValue() != excel.getSysUserId().longValue()) {
			return R.fail("没有权限");
		}

		if (importLogService.removeById(id)) {
			// 删除记录后，根据id在clickhouse中删除对应分区
			importPhoneService.deletePartition(id);

		}
		return R.success("删除成功");
	}
}
