package org.springblade.useranalysis.controller;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springblade.core.tool.api.R;
import org.springblade.useranalysis.common.utils.CommonUtil;
import org.springblade.useranalysis.common.utils.Constants;
import org.springblade.useranalysis.common.utils.ErrorEnum;
import org.springblade.useranalysis.dto.ResultSegment;
import org.springblade.useranalysis.dto.TagHistory;
import org.springblade.useranalysis.entity.AdsUserprofileTag;
import org.springblade.useranalysis.entity.AdsUserprofileTagOrderEntity;
import org.springblade.useranalysis.entity.UserProfileTagView;
import org.springblade.useranalysis.service.TagOrderService;
import org.springblade.useranalysis.service.UserProfileService;
import org.springblade.useranalysis.service.UserTagsService;
import org.springblade.useranalysis.vo.AdsUserprofileTagOrderVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 用户画像模型
 */
@RestController
@RequestMapping("/v1/UserProfile")
@Api(value = "mysql用户标签管理", tags = "用户标签管理")
public class UserProfileContorller {

	@Autowired
	private UserProfileService userProfileService;

	@Autowired
	private UserTagsService userTagsService;

	/**
	 * 添加用户标签
	 */
	@ApiOperation("添加用户标签")
	@ApiOperationSupport(order = 1)
	@RequestMapping(value = "/addUserTag", method = RequestMethod.POST)
	public JSONObject AddUser(@RequestBody AdsUserprofileTag adsUserProfileTagEntity) {
		if (StringUtils.isEmpty(adsUserProfileTagEntity.getTagId())) {
			return CommonUtil.errorJson(ErrorEnum.E_400);
		}
		// Todo 判断重复
		if (userProfileService.save(adsUserProfileTagEntity)) {
			return CommonUtil.successJson();
		} else {
			return CommonUtil.errorJson(ErrorEnum.E_400);
		}
	}

	/**
	 * 删除用户标签
	 */
	@ApiOperation("删除用户标签")
	@ApiOperationSupport(order = 2)
	@RequestMapping(value = "/deleteUserTag", method = RequestMethod.POST)
	public JSONObject deleteUserTag(@RequestParam(value = "id", required = true) int id) {
		if (userProfileService.removeById(id)) {
			return CommonUtil.successJson();
		} else {
			return CommonUtil.errorJson(ErrorEnum.E_10010);
		}
	}

	/**
	 * 修改用户标签
	 */
	@ApiOperation("修改用户标签")
	@ApiOperationSupport(order = 3)
	@RequestMapping(value = "/updateUserTag", method = RequestMethod.POST)
	public JSONObject updateUserTag(@RequestBody AdsUserprofileTag adsUserProfileTagEntity) {
		if (userProfileService.updateById(adsUserProfileTagEntity)) {
			return CommonUtil.successJson();
		} else {
			return CommonUtil.errorJson(ErrorEnum.E_10010);
		}

	}

	/**
	 * 标签列表 —— 根据标签名称进行查询 bingo
	 */
	@ApiOperation("标签列表——根据标签名称进行查询")
	@ApiOperationSupport(order = 4)
	@RequestMapping(value = "/searchUserTag", method = RequestMethod.POST)
	public ResultSegment searchUserTag(@RequestParam(value = "tagname", required = false) String tagname,
			@RequestParam(value = "level_1_id", required = false) String level_1_id,
			@RequestParam(value = "level_2_id", required = false) String level_2_id) {
		JSONObject map = userProfileService.searchUserTag(tagname, level_1_id, level_2_id);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
	}

	/**
	 * 首页查询用户标签视图 bingo
	 */
	@ApiOperation("【用户标签->标签视图->标签详情】 - 查询用户标签视图")
	@ApiOperationSupport(order = 6)
	@RequestMapping(value = "/searchUserTagView", method = RequestMethod.POST)
	public ResultSegment userTagView() {
		JSONObject result = userProfileService.userTagView();
		if (result != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, result);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

	/**
	 * 用户标签 —— 用户全量标签查询
	 */
	@ApiOperation("【用户标签->用户全息档案】 —— 用户全量标签查询")
	@ApiOperationSupport(order = 5)
	@RequestMapping(value = "/userTagSearch", method = RequestMethod.POST)
	@Deprecated
	public ResultSegment userTagSearch(String userId) {
		List<UserProfileTagView> userTagList = userTagsService.userTagSearch(userId);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, userTagList);
	}

	/**
	 * 用户标签流量 —— 各维度历史流量查询
	 */
	@ApiOperation("【用户标签->用户全息档案->单个标签统计】——各维度历史流量查询")
	@ApiOperationSupport(order = 7)
	@RequestMapping(value = "/userTagFlowHisSearch", method = RequestMethod.POST)
	@Deprecated
	public ResultSegment userTagFlowHisSearch(@RequestParam String userId, @RequestParam String tagId,
			@RequestParam String start_month, @RequestParam String end_month) throws ParseException {
		start_month = start_month.replace("-", "");
		end_month = end_month.replace("-", "");
		List<TagHistory> maps = null;//userTagsService.userTagFlowHisSearch(userId, tagId, start_month, end_month);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, maps);
	}

	/**
	 * @return
	 */
	@ApiOperation("用户标签 - 历史数据列表查询")
	@ApiOperationSupport(order = 7)
	@RequestMapping(value = "/userTagHisSearch", method = RequestMethod.POST)
	@Deprecated
	public ResultSegment userTagHisSearch(@RequestParam String userId, @RequestParam String tagId,
			@RequestParam String start_month, @RequestParam String end_month) throws ParseException {
		start_month = start_month.replace("-", "");
		end_month = end_month.replace("-", "");
		List<JSONObject> maps = null;//userTagsService.userTagHisSearch(userId, tagId, start_month, end_month);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, maps);
	}

	/**
	 * 标签列表 —— 根据标签ID进行查询 bingo
	 */
	@ApiOperation("【用户标签->标签视图->标签详情】- 根据标签ID进行查询")
	@ApiOperationSupport(order = 8)
	@RequestMapping(value = "/searchTagId", method = RequestMethod.POST)
	public ResultSegment searchTagId(@RequestParam(value = "tagId", required = false) String tagId) {
		JSONObject result = userProfileService.searchTagId(tagId);
		if (result != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, result);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

	/**
	 * 标签数量校验 bingo
	 */
	@ApiOperation("【用户标签->标签视图->标签详情】- 标签数量校验")
	@ApiOperationSupport(order = 9)
	@RequestMapping(value = "/checkTagNum", method = RequestMethod.POST)
	public ResultSegment checkTagNum(@RequestParam(value = "tagId", required = false) String tagId,
			@RequestParam(value = "startDate", required = false) String startDate,
			@RequestParam(value = "endDate", required = false) String endDate) {
		if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
			startDate = startDate.replaceAll("\\-", "");
			endDate = endDate.replaceAll("\\-", "");
		}
		JSONObject result = userTagsService.checkTagNum(tagId, startDate, endDate);
		if (result != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, result);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

	/**
	 * 同二级大类下各标签所占比例 bingo
	 */
	@ApiOperation("【用户标签->标签视图->标签详情】 - 同二级大类下各标签所占比例")
	@ApiOperationSupport(order = 10)
	@RequestMapping(value = "/checkTagRatio", method = RequestMethod.POST)
	public ResultSegment checkTagRatio(@RequestParam(value = "tagId", required = false) String tagId,
			@RequestParam(value = "dataDate", required = false) String dataDate) {
		if (StringUtils.isNotBlank(dataDate)) {
			dataDate = dataDate.replaceAll("\\-", "");
		}
		JSONObject result = userTagsService.checkTagRatio(tagId, dataDate);
		if (result != null) {
			return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, result);
		} else {
			return new ResultSegment(false, ErrorEnum.E_400.getErrorCode(), ErrorEnum.E_400.getErrorMsg());
		}
	}

	/**
	 * 标签列表 —— 展示 bingo
	 */
	@ApiOperation("标签列表 —— 展示")
	@ApiOperationSupport(order = 11)
	@RequestMapping(value = "/UserTagList", method = RequestMethod.POST)
	public ResultSegment listuserTag(@RequestParam("pageNo") int pageNo, @RequestParam("pageSize") int pageSize,
			@RequestParam(value = "level_1_id", required = false) String level_1_id,
			@RequestParam(value = "level_2_id", required = false) String level_2_id) {
		// Page<AdsUserProfileTagEntity> userPage = new Page<>(pageNo, pageSize);
		IPage<AdsUserprofileTag> userPage = new Page<AdsUserprofileTag>(pageNo, pageSize);
		// 构建条件
		QueryWrapper<AdsUserprofileTag> wrapper = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(level_1_id))
			wrapper.eq("level_1_id", level_1_id);
		if (StringUtils.isNotEmpty(level_2_id))
			wrapper.eq("level_2_id", level_2_id);
		JSONObject map = userProfileService.listUserTag(userPage, wrapper);
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, map);
	}

	/**
	 * 一级标签列表 —— 展示 bingo
	 */

	@ApiOperation("一级标签列表 —— 展示")
	@ApiOperationSupport(order = 12)
	@RequestMapping(value = "/V1TagList", method = RequestMethod.POST)
	public ResultSegment listV1TagList() {
		List<Map<String, Object>> maps = userProfileService.listMaps(new QueryWrapper<AdsUserprofileTag>()
				.select("level_1_id", "level_1_name").groupBy("level_1_id", "level_1_name"));
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, maps);
	}

	/**
	 * 二级标签列表 —— 展示 bingo
	 */

	@ApiOperation("二级标签列表 —— 展示")
	@ApiOperationSupport(order = 13)
	@RequestMapping(value = "/V2TagList", method = RequestMethod.POST)
	public ResultSegment listV2TagList() {
		List<Map<String, Object>> maps = userProfileService.listMaps(new QueryWrapper<AdsUserprofileTag>()
				.select("level_2_id", "level_2_name").groupBy("level_2_id", "level_2_name"));
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, maps);
	}
	
	/**
	 * 
	 * @return
	 */
	@ApiOperation(value = "调整标签值的顺序", notes = "只需要传递2个参数值即可：id和tagOrder")
	@ApiOperationSupport(order = 16)
	@RequestMapping(value = "/adjustTagOrder", method = RequestMethod.POST)
	public R<String> adjustTagOrder(@RequestBody List<AdsUserprofileTag> list){
		return R.status(userProfileService.updateBatchById(list));
	}

	/**
	 * 自定义标签顺序
	 */

	@Autowired
	private TagOrderService tagOrderService;

	@ApiOperation("自定义标签顺序")
	@ApiOperationSupport(order = 14)
	@RequestMapping(value = "/orderTagList", method = RequestMethod.POST)
	public JSONObject orderTagList(@RequestBody List<AdsUserprofileTagOrderEntity> adsUserprofileTagOrderEntities) {
		if (adsUserprofileTagOrderEntities == null || adsUserprofileTagOrderEntities.size() == 0) {
			return CommonUtil.errorJson(ErrorEnum.E_400);
		}
		// 全量删除+全量更新
		if (tagOrderService.clear() && tagOrderService.saveBatch(adsUserprofileTagOrderEntities)) {
			return CommonUtil.successJson();
		} else {
			return CommonUtil.errorJson(ErrorEnum.E_400);
		}
	}

	@ApiOperation("自定义标签查询")
	@ApiOperationSupport(order = 15)
	@RequestMapping(value = "/searchOrderTagList", method = RequestMethod.POST)
	public ResultSegment searchOrderTagList() {
		List<AdsUserprofileTagOrderEntity> orderList = tagOrderService.list(new QueryWrapper<>());
		Map<String, AdsUserprofileTagOrderEntity> userTagOrderMap = orderList.stream().collect(Collectors.toMap(AdsUserprofileTagOrderEntity::getTagName, t -> t, (m1, m2) -> m1));
		List<Map<String, Object>> mapList = userProfileService.listMaps(new QueryWrapper<AdsUserprofileTag>()
				.select("english_name", "level_2_name").groupBy("english_name", "level_2_name"));
		List<AdsUserprofileTagOrderVO> reslist = new ArrayList<>();
		mapList.forEach(item -> {
			String englishName = item.get("english_name").toString();
			String leve2Name = item.get("level_2_name").toString();
			AdsUserprofileTagOrderVO entity = new AdsUserprofileTagOrderVO();
			
			AdsUserprofileTagOrderEntity tagOrderEntity = userTagOrderMap.get(englishName);
			entity.setTagOrder(tagOrderEntity == null ? 0 : tagOrderEntity.getTagOrder());
			entity.setTagDesc(tagOrderEntity == null ? "" : tagOrderEntity.getTagDesc());
			entity.setTagName(englishName);
			entity.setCName(leve2Name);
			reslist.add(entity);
		});
		reslist.sort(Comparator.comparing(AdsUserprofileTagOrderVO::getTagOrder));
//		Map<String, String> map = new HashMap<>();
//		maps.forEach(v -> {
//			map.put((String) v.get("english_name"), (String) v.get("level_2_name"));
//		});
//		List<AdsUserprofileTagOrderVO> reslist = new ArrayList<>();
//		list.forEach(i -> {
//			AdsUserprofileTagOrderVO a = new AdsUserprofileTagOrderVO();
//			a.setTagOrder(i.getTagOrder());
//			a.setTagName(i.getTagName());
//			a.setCName((map.containsKey(i.getTagName())) ? map.get(i.getTagName()) : "");
//			reslist.add(a);
//		});
		return new ResultSegment(true, Constants.SUCCESS_CODE, Constants.SUCCESS_MSG, reslist);

	}

}
