/**
 * 
 */
package org.springblade.useranalysis.controller;

import java.text.ParseException;
import java.util.List;

import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.useranalysis.dto.TagHistory;
import org.springblade.useranalysis.entity.AdsUserprofileTag;
import org.springblade.useranalysis.entity.DimChannelInfo;
import org.springblade.useranalysis.entity.DimProductInfo;
import org.springblade.useranalysis.entity.DimProvinceArea;
import org.springblade.useranalysis.service.UserProfileService;
import org.springblade.useranalysis.service.UserTagsService;
import org.springblade.useranalysis.service.normal.DimChannelInfoService;
import org.springblade.useranalysis.service.normal.DimProductInfoService;
import org.springblade.useranalysis.service.normal.DimProvinceAreaService;
import org.springblade.useranalysis.service.normal.NewUserTagsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * @author wookvn
 *
 */
@RestController
@RequestMapping("/v2/userProfile")
@Api(value = "[新版]用户标签管理", tags = "[新版]用户标签管理")
public class UserProfileNewController extends BladeController {

	@Autowired
	private DimProductInfoService dimProductInfoService;
	
	@Autowired
	private DimProvinceAreaService dimProvinceAreaService;
	
	@Autowired
	private DimChannelInfoService dimChannelInfoService;
	
	@Autowired
	private UserProfileService userProfileService;
	
	@Autowired
	private UserTagsService userTagsService;
	
	@Autowired
	private NewUserTagsService newUserTagsService;
	
	/**
	 * 订购产品三级联动数据列表
	 */
	@ApiOperation("订购产品三级联动数据列表")
	@ApiOperationSupport(order = 1)
	@PostMapping("/spList")
	public R<List<DimProductInfo>> spList() {
		return R.data(dimProductInfoService.list());
	}
	
	/**
	 * 城市标签的二级联动数据列表
	 */
	@ApiOperation("城市标签的二级联动数据列表")
	@ApiOperationSupport(order = 2)
	@PostMapping("/paList")
	public R<List<DimProvinceArea>> paList() {
		return R.data(dimProvinceAreaService.list());
	}
	
	/**
	 * 订购渠道标签的二级联动数据列表
	 */
	@ApiOperation("订购渠道标签的二级联动数据列表")
	@ApiOperationSupport(order = 3)
	@PostMapping("/channelList")
	public R<List<DimChannelInfo>> channelList() {
		return R.data(dimChannelInfoService.list());
	}
	
	/**
	 * 
	 * @return
	 */
	@ApiOperation(value = "仅获取一级和二级标签基本信息列表", notes = "主要用于用户人群分析页面，选择维度")
	@ApiOperationSupport(order = 4)
	@PostMapping("/level1_2List")
	public R<List<AdsUserprofileTag>> level1_2List(){
		return R.data(userProfileService.getLevel1_2List());
	}
	
	/**
	 * 
	 * @param level2Id
	 * @param userId
	 * @return
	 */
	@ApiOperation(value = "根据手机号查询历史数据记录", notes = "全息档案-APP使用图表展示")
	@ApiOperationSupport(order = 5)
	@PostMapping("/userTagHisDataListByUserId")
	public R<List<JSONObject>> userTagHisDataListByUserId(String userId, String level2Id, String start_month, String end_month){
		start_month = start_month != null ? start_month.replace("-", "") : null;
		end_month = end_month != null ? end_month.replace("-", "") : null;
		return R.data(userTagsService.userTagHisDataListByUserId(userId, level2Id, start_month, end_month));
	}
	
	/**
	 */
	@ApiOperation(value = "根据手机号查询月消费等情况", notes = "全息档案-月消费/月流量消耗/月视频流量消耗")
	@ApiOperationSupport(order = 6)
	@PostMapping("/userTagFlowHisSearch")
	public R<List<TagHistory>> userTagFlowHisSearch(@RequestParam String userId, @RequestParam String level2Id,
			@RequestParam String start_month, @RequestParam String end_month) throws ParseException {
		start_month = start_month.replace("-", "");
		end_month = end_month.replace("-", "");
		List<TagHistory> maps = userTagsService.userTagFlowHisSearch(userId, level2Id, start_month, end_month);
		return R.data(maps);
	}
	

	/**
	 * @return
	 */
	@ApiOperation(value = "根据手机号查询近半年产品订购情况", notes = "全息档案-近半年产品订购情况")
	@ApiOperationSupport(order = 7)
	@PostMapping("/userTagHisSearch")
	public R<List<JSONObject>> userTagHisSearch(@RequestParam String userId, @RequestParam String level2Id,
			@RequestParam String start_month, @RequestParam String end_month) throws ParseException {
//		start_month = start_month.replace("-", "");
//		end_month = end_month.replace("-", "");
		List<JSONObject> maps = newUserTagsService.userTagProductHisSearch(userId, start_month, end_month);
		return R.data(maps);
	}
	
	/**
	 * 根据手机号查询产品列表
	 * @param userId
	 * @return
	 */
	@ApiOperation(value = "根据用户手机号查询产品标签最新数据", notes = "用于全息档案-产品标签列表展示")
	@ApiOperationSupport(order = 8)
	@PostMapping("/productListByUserId")
	public R<JSONObject> productListByUserId(String userId) {
		return R.data(newUserTagsService.getProductListByUserId(userId));
	}
	
	/**
	 * 
	 * @param userId
	 * @return
	 */
	@ApiOperation(value = "根据用户手机号查询大网标签和算法标签最新数据", notes = "用于全息档案-大网标签和算法标签数据展示")
	@ApiOperationSupport(order = 9)
	@PostMapping("/xxhAlgDataListByUserId")
	public R<List<JSONObject>> xxhAlgDataListByUserId(String userId) {
		return R.data(newUserTagsService.getXxhAlgDataListByUserId(userId));
	}
}
