/**
 * 
 */
package org.springblade.useranalysis.enums;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;

import org.springblade.useranalysis.dto.param.ParamUnit;

/**
 * @author wookvn
 *
 */
public enum RetentionEventUnit {

	day("DAY"){
		@Override
		public ParamUnit calculate(LocalDate startLocalDate, LocalDate endLocalDate, String dateFormat) {
			List<String> dateStartList = new ArrayList<String>();
			List<String> labelList = new ArrayList<String>();
			long distance = ChronoUnit.DAYS.between(startLocalDate, endLocalDate);
		    if (distance < 0) {
		    	throw new RuntimeException("开始日期不能大于结束日期");
		    }
		    Stream.iterate(startLocalDate, d -> d.plusDays(1)).limit(distance + 1).forEach(f -> {
		    	String dateStr = DateTimeFormatter.ofPattern(dateFormat).format(f);
		    	dateStartList.add(dateStr);
		    	labelList.add(dateStr + " " + RetentionEventUnit.day.weekParse(f.getDayOfWeek()));
		    });
			return new ParamUnit(labelList, dateStartList);
		}
	},
	
	week("WEEK"){
		@Override
		public ParamUnit calculate(LocalDate startLocalDate, LocalDate endLocalDate, String dateFormat) {
			List<String> dateStartList = new ArrayList<String>();
			List<String> labelList = new ArrayList<String>();
			LocalDate startDate = startLocalDate.with(DayOfWeek.MONDAY);
			LocalDate endDate = endLocalDate.with(DayOfWeek.SUNDAY);
			long distance = ChronoUnit.WEEKS.between(startDate, endDate);
			if (distance < 0) {
		    	throw new RuntimeException("开始日期不能大于结束日期");
		    }
			Stream.iterate(startDate, d -> d.plusWeeks(1)).limit(distance + 1).forEach(f -> {
		    	String dateStr = DateTimeFormatter.ofPattern(dateFormat).format(f);
		    	dateStartList.add(dateStr);
		    	labelList.add(dateStr + "~" + DateTimeFormatter.ofPattern(dateFormat).format(f.with(DayOfWeek.SUNDAY)));
		    });
			return new ParamUnit(labelList, dateStartList);
		}
	},
	
	month("MONTH"){
		@Override
		public ParamUnit calculate(LocalDate startLocalDate, LocalDate endLocalDate, String dateFormat) {
			List<String> dateStartList = new ArrayList<String>();
			List<String> labelList = new ArrayList<String>();
			//获取最大的开始和结束时间
			LocalDate startDate = startLocalDate.with(TemporalAdjusters.firstDayOfMonth());
			LocalDate endDate = endLocalDate.with(TemporalAdjusters.lastDayOfMonth());
			long distance = ChronoUnit.MONTHS.between(startDate, endDate);
			if (distance < 0) {
		    	throw new RuntimeException("开始日期不能大于结束日期");
		    }
			Stream.iterate(startDate, d -> d.plusMonths(1)).limit(distance + 1).forEach(f -> {
		    	String dateStr = DateTimeFormatter.ofPattern(dateFormat).format(f);
		    	dateStartList.add(dateStr);
		    	labelList.add(dateStr + "~" + DateTimeFormatter.ofPattern(dateFormat).format(f.with(TemporalAdjusters.lastDayOfMonth())));
		    });
			return new ParamUnit(labelList, dateStartList);
		}
	};
	
	public String clickHouseUnit;
	private RetentionEventUnit(String clickHouseUnit) {
		this.clickHouseUnit = clickHouseUnit;
	}
	
	public abstract ParamUnit calculate(LocalDate startLocalDate, LocalDate endLocalDate, String dateFormat);
	
	private String weekParse(DayOfWeek dayOfWeek) {
		switch(dayOfWeek){
			case MONDAY:{return "周一";}
			case TUESDAY:{return "周二";}
			case WEDNESDAY:{return "周三";}
			case THURSDAY:{return "周四";}
			case FRIDAY:{return "周五";}
			case SATURDAY:{return "周六";}
			case SUNDAY:{return "周日";}
		}
		return "未知";
	}
}
