package org.springblade.useranalysis.mapper.clickhouse;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springblade.useranalysis.entity.UserProfileTagView;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;

/**
 * 用户/行为/大宽表
 */

public interface UserTagsMapper extends BaseMapper<ZUserTagEntity> {

//	@Deprecated
//	List<UserProfileTagView> tagView(@Param("userId") String userId, @Param("month") String month);
//	
//	@Deprecated
//	List<UserProfileTagView> tagView1(@Param("userId") String userId);

	List<UserProfileTagView> findTagViewByUserId(@Param("sql") String sql, @Param("userId") String userId);

	@Select("select DISTINCT THEMONTH_V, ${columnC}, ${columnE}, ${columnV} from ads.Z_USER_TAG_FLAT_ALL where USERID_V = #{userId} and ${columnV} is not null and toString(${columnV}) != '' and THEMONTH_V >= #{startMonth} and THEMONTH_V <= #{endMonth} order by THEMONTH_V")
	List<Map<String, Object>> findOnceTag(@Param("userId") String userId, @Param("startMonth") String startMonth,
			@Param("endMonth") String endMonth, @Param("columnC") String columnC, @Param("columnE") String columnE,
			@Param("columnV") String columnV);

	/**
	 * 标签数量校验
	 */
	@Select("select THEMONTH_V as datadate, uniqCombined(USERID_V) as num from ads.Z_USER_TAG_FLAT_ALL ${value} group by THEMONTH_V order by THEMONTH_V")
	List<JSONObject> checkTagNum(String sql);

	/**
	 * 同二级大类下各标签所占比例
	 */
	@Select("select ${columnName} as tagname, uniqCombined(USERID_V) as num from ads.Z_USER_TAG_FLAT_ALL ${whereSql} group by ${columnName} HAVING tagname!='' order by num desc LIMIT 50")
	List<JSONObject> checkTagRatio(@Param("columnName") String columnName, @Param("whereSql") String whereSql);

	/**
	 * 人群数量计算
	 */
	@Select("SELECT count(DISTINCT USERID_V) FROM Z_USER_TAG_FLAT_ALL ${ew.customSqlSegment}") // COUNT(DISTINCT
	Integer userGroupCount(@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper);

	/**
	 * 人群数量计算-基于导入的人群
	 */
	@Select("with toInt32OrZero(THEMONTH_V) as INT_THEMONTH_V\n"
			+ "SELECT\n"
			+ "	count(DISTINCT USERID_V)\n"
			+ "FROM\n"
			+ "	(\n"
			+ "	SELECT\n"
			+ "		USERID_V\n"
			+ "	FROM\n"
			+ "		Z_USER_TAG_FLAT_ALL\n"
			+ "	${ew.customSqlSegment}) z\n"
			+ "inner join (\n"
			+ "	select\n"
			+ "		DISTINCT phone\n"
			+ "	from\n"
			+ "		USER_EXCEL_IMPORT_PHONE_ALL u\n"
			+ "	where\n"
			+ "		id = #{importId}\n"
			+ "	) u on\n"
			+ "	z.USERID_V = u.phone") // COUNT(DISTINCT
	Integer userGroupCountBaseImport(String importId,
			@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper);

	/**
	 * 人群分析 - 基础数据格式 select * from (select tCode,COUNT(1) as num FROM (select
	 * DISTINCT USERID_V,${cName} as tCode from Z_USER_TAG_FLAT_ALL
	 * ${ew.customSqlSegment}) s group by tCode) k order by num desc
	 */
	@Select("select uniqCombined(USERID_V) as num, ${cName} as tCode from Z_USER_TAG_FLAT_ALL ${ew.customSqlSegment} group by tCode order by num desc")
	List<Map<String, Object>> groupAnalysis(@Param("cName") String cName,
			@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper);
	
	@Select("with toInt32OrZero(THEMONTH_V) as INT_THEMONTH_V\n"
			+ "select\n"
			+ "	count(DISTINCT USERID_V) as num,\n"
			+ "	${cName} as tCode\n"
			+ "from\n"
			+ "	(\n"
			+ "	select\n"
			+ "		*\n"
			+ "	from\n"
			+ "		Z_USER_TAG_FLAT_ALL ${ew.customSqlSegment}) z\n"
			+ "inner join (\n"
			+ "	select\n"
			+ "		DISTINCT phone\n"
			+ "	from\n"
			+ "		USER_EXCEL_IMPORT_PHONE_ALL\n"
			+ "	where\n"
			+ "		id = #{importId}\n"
			+ "	) u on\n"
			+ "	z.USERID_V = u.phone\n"
			+ "group by\n"
			+ "	tCode\n"
			+ "order by\n"
			+ "	num desc")
	List<Map<String, Object>> groupAnalysisBaseImport(String importId,@Param("cName") String cName,
			@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper);

	/**
	 * 精准营销推送
	 * 
	 * @param TagWrapper
	 * @return
	 */
	@Select("SELECT DISTINCT USERID_V FROM Z_USER_TAG_FLAT_ALL ${ew.customSqlSegment}")
	List<String> queryUserIdList(@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper);
	
	@Select("SELECT\n"
			+ "	DISTINCT USERID_V\n"
			+ "from\n"
			+ "	(\n"
			+ "	SELECT\n"
			+ "		DISTINCT USERID_V\n"
			+ "	FROM\n"
			+ "		Z_USER_TAG_FLAT_ALL\n"
			+ "	${ew.customSqlSegment}) t1\n"
			+ " inner join (\n"
			+ "	select\n"
			+ "		*\n"
			+ "	from\n"
			+ "		USER_EXCEL_IMPORT_PHONE_ALL\n"
			+ "	where\n"
			+ "		id = #{importId} ) t2\n"
			+ "on\n"
			+ "	t1.USERID_V = t2.phone ${sqlLimit}")
	List<String> queryUserIdListBaseImport(@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper,String importId,String sqlLimit);
	
	@Select("SELECT\n"
			+ "	DISTINCT phone\n"
			+ "from\n"
			+ "	USER_EXCEL_IMPORT_PHONE_ALL\n"
			+ "	where\n"
			+ "		id = #{importId} ${sqlLimit}\n")
	List<String> queryUserIdListFromImport(String importId,String sqlLimit);

	@Deprecated
	@Select("select * from (select  tCode,COUNT(1) as num FROM (select DISTINCT USERID_V,tCode from Z_USER_TAG array join ${cName} as tCode  ${ew.customSqlSegment}) s group by tCode) k order by num desc")
	List<Map<String, Object>> groupAnalysisByArray(@Param("cName") String cName,
			@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper);

//	@Deprecated
//	List<UserProfileTag2Tree> personalTagView(@Param("userId") String userId, @Param("month") String month);

	@Deprecated
	@Select(" Select A.* , du.level_1_name , du.level_2_name from DIC_USERTAG du  LEFT JOIN ( SELECT USERID_V, THEMONTH_V , tId , tCode, tValue FROM Z_USER_TAG array join EVENT_CPNAME.E as tId , EVENT_CPNAME.C as tCode, EVENT_CPNAME.V as tValue ) A on A.tId = du.tag_id")
	List<JSONObject> listClickhouse();

	@Deprecated
	@Select("select THEMONTH_V from ads.Z_USER_TAG where USERID_V =#{userid} order by TAG_AGE_V desc LIMIT 1")
	List<String> getLastMonth(@Param("userid") String userid);

	@Select("SELECT ${fields} FROM Z_USER_TAG_FLAT_ALL ${ew.customSqlSegment}")
	List<Map<String, Object>> exportTagList(@Param("fields") String fields,
			@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper);
	
	@Select("SELECT\n"
			+ "	${fields}\n"
			+ "from\n"
			+ "	(\n"
			+ "	SELECT\n"
			+ "		${fields}\n"
			+ "	FROM\n"
			+ "		Z_USER_TAG_FLAT_ALL\n"
			+ "	${ew.customSqlSegment}) t1\n"
			+ " inner join (\n"
			+ "	select\n"
			+ "		*\n"
			+ "	from\n"
			+ "		USER_EXCEL_IMPORT_PHONE_ALL\n"
			+ "	where\n"
			+ "		id = #{importId} ) t2\n"
			+ "on\n"
			+ "	t1.USERID_V = t2.phone ${sqlLimit}")
	List<Map<String, Object>> exportTagListBaseImport(@Param("fields") String fields,
			@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> TagWrapper,String importId,String sqlLimit);
	
	@Select("SELECT\n"
			+ "	${fields}\n"
			+ "FROM\n"
			+ "	(\n"
			+ "	SELECT\n"
			+ "		${fields}\n"
			+ "	FROM\n"
			+ "		Z_USER_TAG_FLAT_ALL\n"
			+ "	WHERE\n"
			+ "		THEMONTH_V >= #{monthStart}\n"
			+ "		AND THEMONTH_V <= #{monthEnd}\n"
			+ "	) t1\n"
			+ "INNER JOIN (\n"
			+ "	SELECT\n"
			+ "		USERID_V\n"
			+ "	FROM\n"
			+ "		Z_USER_TAG_FLAT_ALL\n"
			+ "	${ew.customSqlSegment}\n"
			+ "	) t2 ON t1.USERID_V = t2.USERID_V\n"
			+ "${sqlLimit}")
	List<Map<String, Object>> exportTagListMonths(
			@Param("fields") String fields,
			String monthStart,
			String monthEnd,
			@Param(Constants.WRAPPER) QueryWrapper<ZUserTagEntity> childWrapper,
			String sqlLimit);
}
