package org.springblade.useranalysis.service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.springblade.useranalysis.common.utils.Tuple;
import org.springblade.useranalysis.dto.EventFilterCondition;
import org.springblade.useranalysis.dto.EventFilterParameter;
import org.springblade.useranalysis.dto.RequestDistributeAnalysis;
import org.springblade.useranalysis.dto.RequestEventAnalysis;
import org.springblade.useranalysis.dto.RequestHopperAnalysis;
import org.springblade.useranalysis.dto.RequestRetentionAnalysis;
import org.springblade.useranalysis.dto.param.ParamProp;
import org.springblade.useranalysis.entity.clickhouse.UserEvent;
import org.springblade.useranalysis.enums.EventConnectType;
import org.springblade.useranalysis.enums.EventFunction;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.IService;

import cn.hutool.core.util.StrUtil;

/**
 * (UserEvent)表服务接口
 *
 * @author limeng
 * @since 2021-07-06 14:38:03
 */
public interface UserEventService extends IService<UserEvent> {

	public JSONObject reaminAnalysis(RequestRetentionAnalysis requestRemainAnalysis) throws ParseException;

	public JSONObject hopperAnalysis(RequestHopperAnalysis requestHopperAnalysis) throws ParseException;

	public JSONObject eventAnalysis(RequestEventAnalysis eventAnalysisRequest);

	public JSONObject distributeAnalysis(RequestDistributeAnalysis distributeAnalysisRequest) throws ParseException;

	public List<String> findAidList(String ptype);

	// 转化百分比
	default String pernt(Object doc) {
		Double double_doc = (Double) doc;
		if (!Double.isNaN(double_doc)) {
			return new BigDecimal(double_doc * 100).setScale(2, BigDecimal.ROUND_HALF_UP).toString() + "%";
		}
		return "0%";
	}

	/**
	 * 封装查询条件
	 * 
	 * @param eventCode
	 * @param eventFilter
	 * @param eventPropList 事件的属性列表
	 */
	default QueryWrapper<UserEvent> buildCondition(String ptype, EventFilterParameter eventFilter,
			List<ParamProp> eventPropList) {
		QueryWrapper<UserEvent> queryWrapper = Wrappers.<UserEvent>query().eq("ptype", ptype);
		if (eventFilter != null && eventFilter.getConditions() != null && !eventFilter.getConditions().isEmpty()
				&& eventPropList != null && !eventPropList.isEmpty()) {
			Map<Long, ParamProp> eventPropMap = eventPropList.stream()
					.collect(Collectors.toMap(ParamProp::getId, Function.identity(), (v1, v2) -> v1));

			// 封装查询条件
			queryWrapper.and(wrapper -> {
				int size = eventFilter.getConditions().size();
				for (int i = 0; i < size; i++) {
					EventFilterCondition condition = eventFilter.getConditions().get(i);
					ParamProp paramProp = eventPropMap.get(condition.getId());
					// 不是isNotNull/isNull，并且无参数，不符合要求条件
					if (paramProp == null || (condition.getFunction() != EventFunction.isNotNull
							&& condition.getFunction() != EventFunction.isNull
							&& (condition.getValues() == null || condition.getValues().isEmpty()))) {
						continue;
					}

//					List<Object> objValues = new ArrayList<Object>();
//					if(condition.getValues() != null && !condition.getValues().isEmpty()) { //数据转换
//						if(condition.getDataType() == EventDataType.number && paramProp.isCommon()) {
//							condition.getValues().forEach(f -> objValues.add(Long.valueOf(f)));
//						}else {
//							condition.getValues().forEach(f -> objValues.add(f));
//						}
//					}
					// 构造查询
					condition.getFunction().buildQuery(wrapper, paramProp.getCode(), condition.getValues(),
							paramProp.isCommon());
					if (eventFilter.getConnectType() == EventConnectType.or && (i != size - 1)) {
						wrapper.or();
					}
				}
			});

		}
		return queryWrapper;
	}

	/**
	 * 维度属性ID转换为维度属性Code
	 * 
	 * @param targetList
	 * @param paramPropList
	 * @return  <code, propId>
	 */
	default List<Tuple<String, Long>> targetPropId2Code(Set<String> targetList, List<ParamProp> paramPropList) {
		List<Tuple<String, Long>> targetPropCodeList = new ArrayList<Tuple<String, Long>>();
		if (targetList != null && paramPropList != null) {
			for (String propIdStr : targetList) {
				Tuple<String, Long> tuple = targetPropId2Code(propIdStr, paramPropList);
				if(tuple != null) {
					targetPropCodeList.add(tuple);
				}
			}
		}
		return targetPropCodeList;
	}

	default Tuple<String, Long> targetPropId2Code(String propIdStr, List<ParamProp> paramPropList) {
		if(StrUtil.isEmpty(propIdStr)) {
			return null;
		}
		Long propId = Long.valueOf(propIdStr);
		Optional<ParamProp> optional = paramPropList.stream().filter(f -> f.getId().longValue() == propId.longValue())
				.findFirst();
		if (optional.isPresent()) {
			return new Tuple<String, Long>(optional.get().getCode(), Long.valueOf(propIdStr));
		}
		return null;
	}
}
