package org.springblade.useranalysis.service.impl;


import java.util.List;
import java.util.Map;

import org.springblade.useranalysis.entity.AdsEventList;
import org.springblade.useranalysis.entity.AdsPropertyFilter;
import org.springblade.useranalysis.entity.AdsPropertyList;
import org.springblade.useranalysis.mapper.AdsUserprofileTagMapper;
import org.springblade.useranalysis.mapper.EventListMapper;
import org.springblade.useranalysis.service.AdsPropertyFilterService;
import org.springblade.useranalysis.service.AdsPropertyListService;
import org.springblade.useranalysis.service.EventListService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 行为事件下拉菜单
 */


@Service
@DS("user_analysis_db")
public class EventListServiceImpl extends ServiceImpl<EventListMapper, AdsEventList> implements EventListService {

    @Autowired
    private AdsPropertyListService adsPropertyListService;

    @Autowired
    private AdsPropertyFilterService adsPropertyFilterService;

    @Autowired
    private AdsUserprofileTagMapper adsUserprofileTagMapper;

    /**
     * 行为事件下拉菜单
     */
    @Override
    public List<Map<String, Object>> eventList() {
        List<Map<String, Object>> eventList = baseMapper.selectMaps(new QueryWrapper<AdsEventList>());
        return eventList;
    }


    /**
     * 细分维度[设备属性/用户属性] 下拉菜单
     */

    @Override
    public List<Map<String, Object>> attributeList() {
        List<Map<String, Object>> attributeList = adsPropertyListService.listMaps(new QueryWrapper<AdsPropertyList>());
        return attributeList;
    }


    /**
     * 用户标签下拉菜单 标签筛选用户标签下拉1 2 3级菜单
     */

    @Override
    public JSONObject dropDownList1(String level1) {
        JSONObject returnResult = new JSONObject();
        List<JSONObject> list = adsUserprofileTagMapper.dropDownList1(level1);
        returnResult.put("list", list);
        returnResult.put("total", list.size());
        return returnResult;
    }

    @Override
    public JSONObject dropDownList2(String level1, String level2) {
        JSONObject returnResult = new JSONObject();
        List<JSONObject> list = adsUserprofileTagMapper.dropDownList2(level1, level2);
        returnResult.put("list", list);
        returnResult.put("total", list.size());
        return returnResult;
    }


    /**
     * 用户行为分析 下拉菜单过滤条件筛选Key Value
     */

    @Override
    public JSONObject dropDownListKey() {
        QueryWrapper<AdsPropertyFilter> eventPropertyFilterEntityQueryWrapper = new QueryWrapper<>();
        eventPropertyFilterEntityQueryWrapper.select("level_2_name AS name,level_2_id AS field").groupBy("level_2_name", "level_2_id");
        List<Map<String, Object>> list= adsPropertyFilterService.listMaps(eventPropertyFilterEntityQueryWrapper);
        JSONObject returnResult = new JSONObject();
        returnResult.put("list", list);
        returnResult.put("total", list.size());
        return returnResult;
    }

    @Override
    public JSONObject dropDownListValue(String level_2_name) {
        JSONObject returnResult = new JSONObject();
        QueryWrapper<AdsPropertyFilter> eventPropertyFilterEntityQueryWrapper = new QueryWrapper<>();
        eventPropertyFilterEntityQueryWrapper.select("property_type_code AS type,level_2_id AS field,tag_name AS name,tag_id AS tag")
                .eq("level_2_name", level_2_name)
                .groupBy("property_type_code", "level_2_id", "tag_name", "tag_id");
        List<Map<String, Object>> list= adsPropertyFilterService.listMaps(eventPropertyFilterEntityQueryWrapper);

        returnResult.put("list", list);
        returnResult.put("total", list.size());
        return returnResult;
    }

    @Override
    public JSONObject dropDownList() {
        JSONObject returnResult = new JSONObject();

        QueryWrapper<AdsPropertyFilter> eventPropertyFilterEntityQueryWrapper = new QueryWrapper<>();
        eventPropertyFilterEntityQueryWrapper.select("level_2_name AS filter,property_type_code AS type,level_2_id AS field,tag_name AS name,tag_id AS tag")
                .groupBy("level_2_name", "property_type_code", "level_2_id", "tag_name", "tag_id");

        List<Map<String, Object>> list= adsPropertyFilterService.listMaps(eventPropertyFilterEntityQueryWrapper);
        returnResult.put("list", list);
        returnResult.put("total", list.size());
        return returnResult;
    }

}
