package org.springblade.useranalysis.service.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springblade.useranalysis.entity.AdsUserprofileTag;
import org.springblade.useranalysis.entity.TagColumn;
import org.springblade.useranalysis.entity.UserGroupAnalysisRequest;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;
import org.springblade.useranalysis.mapper.UserGroupMapper;
import org.springblade.useranalysis.mapper.clickhouse.UserTagsMapper;
import org.springblade.useranalysis.service.UserGroupAnalysisService;
import org.springblade.useranalysis.service.UserGroupCalculateService;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.UserProfileService;
import org.springblade.useranalysis.service.normal.UserGroupRuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONReader;
import com.alibaba.fastjson2.JSONReader.Feature;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import lombok.extern.slf4j.Slf4j;

/**
 * 用户人群分析模型
 **/
@Slf4j
@Service
@DS("clickhouse")
public class UserGroupAnalysisServiceImpl extends ServiceImpl<UserGroupMapper, UserGroupEntity>
		implements UserGroupAnalysisService {

	@Autowired
	private UserProfileService userProfileService;

	@Autowired
	private UserTagsMapper userTagsMapper;

	@Autowired
	private UserGroupCalculateService userGroupCalculateService;

	@Autowired
	private UserGroupService userGroupService;
	
	@Autowired
	private UserGroupRuleService userGroupRuleService;

	/**
	 * 用户画像 —— 人群多维分析 ok
	 * 
	 * @Parma ruleInfo 筛选人群规则json
	 *        例：{"relationship":"and","ruledetails":[{"key":"A112_08_003","value":""},{"key":"A112_06_004","value":""}]}
	 * @Parma level2Id= Level_2_name
	 */
	public JSONObject parseRule(String importId, UserGroupEntity userGroup, String level2Id) {
		boolean isImport = StringUtils.isNotBlank(importId);
		AdsUserprofileTag userprofileTag = userProfileService.getOne(
				Wrappers.<AdsUserprofileTag>lambdaQuery().eq(AdsUserprofileTag::getLevel2Id, level2Id).last("limit 1"));
		if (userprofileTag == null) {
			log.warn("Invalid level2Id=" + level2Id);
			return null;
		}

		String weiduId = userprofileTag.getTagId();
		TagColumn weiduColumn = userProfileService.getColumName(weiduId, userprofileTag.getLevel2Name());

		List<Map<String, Object>> maps = new ArrayList<>();
		// SQL构造器
//		QueryWrapper<ZUserTagEntity> queryWrapper = userGroupCalculateService.zUserTagParseRule(ruleJson);
		QueryWrapper<ZUserTagEntity> queryWrapper = userGroupRuleService.parse(userGroup);
		// 过滤掉空值
		if (weiduColumn.columnType.equals("array")) {
			if (isImport) {
				maps = userTagsMapper.groupAnalysisBaseImport(importId, weiduColumn.columnC.toUpperCase(),
						queryWrapper);
			} else {
				queryWrapper.ne("tCode", "");
				maps = userTagsMapper.groupAnalysis(weiduColumn.columnC.toUpperCase(), queryWrapper);
			}
		} else {
			if (isImport) {
				maps = userTagsMapper.groupAnalysisBaseImport(importId, "`" + weiduColumn.columnC.toUpperCase() + "`",
						queryWrapper);
			} else {
				queryWrapper.ne("tCode", "");
				maps = userTagsMapper.groupAnalysis("`" + weiduColumn.columnC.toUpperCase() + "`", queryWrapper);
			}
		}
		Map<String, Object> ysdata = new LinkedHashMap<>();
		int topk = 30;
		for (int i = 0; i < maps.size(); i++) {
			ysdata.put((String) maps.get(i).get("tCode"), maps.get(i).get("num"));
			if (i >= topk) {
				break;
			}
		}
//		JSONObject result = JSON.parseObject(JSON.toJSONString(ysdata), JSONReader.Feature.);
		JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(ysdata));
		return result;
	}

	@Override
	public JSONObject usergroupCalculate(UserGroupAnalysisRequest userGroupAnalysisRequest) {
		// 筛选人群标签
		UserGroupEntity userGroup = userGroupService.getById(userGroupAnalysisRequest.getGroupId());
		String importId = userGroup.getImportId();
//		JSONObject usergroupRule = userProfileService.usergroupRule(userGroupAnalysisRequest.getGroupId());
		JSONObject returnResult = parseRule(importId, userGroup, userGroupAnalysisRequest.getLevel2Id());
		return returnResult;
	}

}
