package org.springblade.useranalysis.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springblade.useranalysis.common.utils.CommonUtil;
import org.springblade.useranalysis.entity.AdsUserprofileTag;
import org.springblade.useranalysis.entity.AdsUserprofileTagOrderEntity;
import org.springblade.useranalysis.entity.TagColumn;
import org.springblade.useranalysis.mapper.UserProfileMapper;
import org.springblade.useranalysis.service.TagOrderService;
import org.springblade.useranalysis.service.UserProfileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 用户画像—— 用户标签模块
 */

@Service
@DS("user_analysis_db")
public class UserProfileServiceImpl extends ServiceImpl<UserProfileMapper, AdsUserprofileTag>
		implements UserProfileService {
	
	@Autowired
	private TagOrderService tagOrderService;
	
	/**
	 * 首页查询用户标签视图,展开是树形结构
	 */
	@Override
	public JSONObject userTagView() {
		List<JSONObject> userTagView = baseMapper.userTagView();
		JSONObject map = new JSONObject();
		// 添加树形结构的头部信息
//        map.put("用户标签", userTagView);
//        String label = "{\"position\":\"right\"}";
		map.put("name", "用户标签");
//        map.put("label", JSONObject.parse(label));
		map.put("children", userTagView);

//        System.out.println(map);
		return map;
	}

	/**
	 * 根据标签ID进行查询
	 */
	@Override
	public JSONObject searchTagId(String tagId) {
		List<AdsUserprofileTag> usertaglist = baseMapper.selectList(new QueryWrapper<AdsUserprofileTag>().eq("tag_id", tagId));
		usertaglist.stream().map(item -> {
			formatAbsUPTE(item);
			AdsUserprofileTagOrderEntity tagOrderEntity = tagOrderService.getOne(Wrappers.<AdsUserprofileTagOrderEntity>lambdaQuery().eq(AdsUserprofileTagOrderEntity::getTagName, item.getEnglishName()));
			if(tagOrderEntity != null) {
				item.setLevel2TagDesc(tagOrderEntity.getTagDesc());
			}
			return item;
		});
		JSONObject returnResult = new JSONObject();
		returnResult.put("list", usertaglist);
		returnResult.put("total", returnResult.size());
		return returnResult;
	}

	// 字段转化
	public AdsUserprofileTag formatAbsUPTE(AdsUserprofileTag i) {
		if (i.getTagType().equals("1")) {
			i.setTagType("分类");
		} else {
			i.setTagType("统计");
		}
		if (i.getExclusion() != null && i.getExclusion().equals("1")) {
			i.setExclusion("互斥");
		} else {
			i.setExclusion("非互斥");
		}
		if (i.getIsOnline() != null && i.getIsOnline().equals("1")) {
			i.setIsOnline("使用中");
		} else {
			i.setIsOnline("已停用");
		}
		return i;

	}
	
	@Override
	public TagColumn getColumNameByLevel2Name(String level2Name) {
		TagColumn cName = new TagColumn();
		List<JSONObject> tagNumList = baseMapper.getColumNameByLeve2Name(level2Name);
		if (tagNumList.size() > 0) {
			cName = _buildTagColumnsOfJSONObject(tagNumList).get(0);
		}
		return cName;
	}

	@Override
	public TagColumn getColumNameExport(String level2Id) {
		TagColumn cName = new TagColumn();
		List<JSONObject> tagNumList = baseMapper.getColumNameExport(level2Id);
		if (tagNumList.size() > 0) {
			cName = _buildTagColumnsOfJSONObject(tagNumList).get(0);
		}
		return cName;
	}

	/**
	 * 根据标签Code 获取列的名称
	 *
	 * @param tagId A112_02_002
	 * @return
	 */
	@Override
	public TagColumn getColumName(String tagId, String tagName) {
		TagColumn cName = new TagColumn();
		List<JSONObject> tagNumList = baseMapper.getColumName(tagId);
		if(tagName != null && !"".equals(tagName) && tagNumList != null) {
			tagNumList.removeIf(f -> !f.getString("level_2_name").equals(tagName));
		}
		if (tagNumList.size() > 0) {
			cName = _buildTagColumnsOfJSONObject(tagNumList).get(0);
		}
		return cName;
	}

	/**
	 * 新增用户标签
	 */

	/**
	 * 删除用户标签
	 */
	@Override
	public JSONObject deleteUserTag(String tag_id) {
		baseMapper.deleteById(tag_id);
		return CommonUtil.successJson();
	}

	/**
	 * 根据标签名称进行查询
	 */
	@Override
	public JSONObject searchUserTag(String tagname, String level_1_id, String level_2_id) {
		QueryWrapper<AdsUserprofileTag> queryWrapper = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(tagname))
			queryWrapper.like("tag_name", tagname);
		if (StringUtils.isNotEmpty(level_1_id))
			queryWrapper.eq("level_1_id", level_1_id);
		if (StringUtils.isNotEmpty(level_2_id))
			queryWrapper.eq("level_2_id", level_2_id);
		List<AdsUserprofileTag> adsUserProfileTagEntities = baseMapper.selectList(queryWrapper);
		adsUserProfileTagEntities.stream().map(i -> {
			formatAbsUPTE(i);
			return i;
		});
		JSONObject returnResult = new JSONObject();
		returnResult.put("list", adsUserProfileTagEntities);
		returnResult.put("total", adsUserProfileTagEntities.size());
		return returnResult;
	}

	/**
	 * 用户标签列表 分页展示
	 */
	@Override
	public JSONObject listUserTag(IPage<AdsUserprofileTag> userPage, QueryWrapper<AdsUserprofileTag> wrapper) {

		int count = baseMapper.selectCount(new LambdaQueryWrapper<>());

		IPage<AdsUserprofileTag> mapIPage = baseMapper.selectPage(userPage, wrapper);
		mapIPage.getRecords().stream().map(i -> {
			formatAbsUPTE(i);
			return i;
		});
		List<JSONObject> res = new ArrayList<>();
		mapIPage.getRecords().forEach(i -> {
			res.add(JSONObject.parseObject(JSONObject.toJSONString(i)));
		});

		return CommonUtil.successPageTag((int) mapIPage.getPages(), (int) mapIPage.getSize(), res, count);
	}

	/* 获取全部标签 */
	@Override
	public Map<String, List<AdsUserprofileTag>> getAllUserTag() {

		LambdaQueryWrapper lambdaQueryWrapper = new LambdaQueryWrapper();

		List<AdsUserprofileTag> tagList = baseMapper.selectList(lambdaQueryWrapper);
		Map<String, List<AdsUserprofileTag>> codeMaps = new HashMap<>();

		return codeMaps;
	}

	/**
	 * 用户画像 —— 人群多维分析— 下拉菜单 筛选分析维度
	 */
	@Override
	public JSONObject usergroupAnalysisList() {
		List<JSONObject> dimension = baseMapper.usergroupAnalysisList();
		return CommonUtil.successPage(dimension);
	}

	/**
	 * 用户画像 —— 人群多维分析 —— 填写标签权重值
	 */
	@Override
	public JSONObject usergroupAnalysisTags(String level_2_name, String tag_type) {
		List<JSONObject> tags = new ArrayList<>();
		if (tag_type.equals("1")) {
			tags = baseMapper.usergroupAnalysisTheme(level_2_name);
		} else {
			tags = baseMapper.usergroupAnalysisTag(level_2_name);
		}
		return CommonUtil.successPage(tags);
	}

	/**
	 * 用户画像 —— 人群多维分析-选择人群
	 */
	@Override
	public List<JSONObject> distinctGroupname() {
		List<JSONObject> groupNames = baseMapper.distinctGroupname();
		return groupNames;
	}

	/**
	 * 用户画像 —— 人群多维分析-根据人群id查询人群规则
	 */
	@Override
	public JSONObject usergroupRule(String groupId) {
		String groupRule = baseMapper.usergroupRule(groupId);
		return JSONObject.parseObject(groupRule);
	}

	/**
	 * 用户画像 —— 人群多维分析-根据分析维度查询包含的标签
	 */
	@Override
	public List<String> analysisTags(String dimension) {
		List<String> groupRule = baseMapper.analysisTags(dimension);
		return groupRule;
	}

	@Override
	public List<TagColumn> getAllOnlineTagColumn() {
		LambdaQueryWrapper<AdsUserprofileTag> queryWrapper = Wrappers.<AdsUserprofileTag>lambdaQuery()
				.select(AdsUserprofileTag::getEnglishName, AdsUserprofileTag::getTagCType)
				.eq(AdsUserprofileTag::getIsOnline, "1")
				.groupBy(AdsUserprofileTag::getEnglishName, AdsUserprofileTag::getTagCType);
		List<AdsUserprofileTag> list = baseMapper.selectList(queryWrapper);
		//全息档案不查询未订购标签
		list.removeIf(f -> f.getEnglishName().equalsIgnoreCase("event_unspname"));
		return _buildTagColumns(list);
	}

	@Override
	public List<AdsUserprofileTag> getLevel1_2List() {
		return baseMapper.getLevel1_2List();
	}
	@Override
	public List<TagColumn> getTagColumnList(AdsUserprofileTag userprofileTag) {
		QueryWrapper<AdsUserprofileTag> queryWrapper = new QueryWrapper<AdsUserprofileTag>(userprofileTag);
		queryWrapper.lambda().select(AdsUserprofileTag::getEnglishName, AdsUserprofileTag::getTagCType, AdsUserprofileTag::getSearchType)
		.groupBy(AdsUserprofileTag::getEnglishName, AdsUserprofileTag::getTagCType, AdsUserprofileTag::getSearchType);
		List<AdsUserprofileTag> list = baseMapper.selectList(queryWrapper);
		return _buildTagColumns(list);
	}
	
	@Override
	public List<JSONObject> getProductTagList() {
		return baseMapper.getProductTagList();
	}
	
	private List<TagColumn> _buildTagColumns(List<AdsUserprofileTag> list){
		List<TagColumn> resultList = new ArrayList<TagColumn>();
		for (AdsUserprofileTag profileTag : list) {
			TagColumn tagColumn = new TagColumn();
			String column = profileTag.getEnglishName().toUpperCase();
			// 标签列的类别
			tagColumn.columnType = profileTag.getTagCType();
			// 标签Code
			tagColumn.columnE = profileTag.getTagCType().equalsIgnoreCase("string") ? column : column + ".E";
			// 标签类别名称
			tagColumn.columnC = profileTag.getTagCType().equalsIgnoreCase("string") ? column + "_C" : column + ".C";
			// 标签值
			tagColumn.columnV = profileTag.getTagCType().equalsIgnoreCase("string") ? column + "_V" : column + ".V";
			// 英文名
			tagColumn.englishName = column;
			tagColumn.showHistory = profileTag.getShowHis() != null ? profileTag.getShowHis().toString() : null;
			tagColumn.cnName = profileTag.getLevel2Name();
			tagColumn.searchType = profileTag.getSearchType();
			resultList.add(tagColumn);
		}
		return resultList;
	}
	
	private List<TagColumn> _buildTagColumnsOfJSONObject(List<JSONObject> list){
		List<TagColumn> resultList = new ArrayList<TagColumn>();
		for(JSONObject jsonObject : list) {
			String ctype = jsonObject.getString("tag_c_type");
			String column = jsonObject.getString("english_name");
			String show_his = jsonObject.getString("show_his");
			String cnName = jsonObject.getString("level_2_name");
			
			TagColumn tagColumn = new TagColumn();
			// 标签列的类别
			tagColumn.columnType = ctype;
			tagColumn.showHistory = show_his;
			tagColumn.cnName = cnName;
			// 标签Code
			tagColumn.columnE = ctype.equalsIgnoreCase("string") ? column : column + ".E";
			// 标签类别名称
			tagColumn.columnC = ctype.equalsIgnoreCase("string") ? column + "_C" : column + ".C";
			// 标签值
			tagColumn.columnV = ctype.equalsIgnoreCase("string") ? column + "_V" : column + ".V";
			// 英文名
			tagColumn.englishName = column;
			resultList.add(tagColumn);
		}
		return resultList;
	}

}
