package org.springblade.useranalysis.service.impl;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springblade.useranalysis.dto.TagHistory;
import org.springblade.useranalysis.entity.AdsUserprofileTag;
import org.springblade.useranalysis.entity.TagColumn;
import org.springblade.useranalysis.entity.UserProfileTagItem;
import org.springblade.useranalysis.entity.UserProfileTagView;
import org.springblade.useranalysis.entity.clickhouse.ZUserTagEntity;
import org.springblade.useranalysis.mapper.clickhouse.DynamicQueryMapper;
import org.springblade.useranalysis.mapper.clickhouse.UserTagsMapper;
import org.springblade.useranalysis.service.UserProfileService;
import org.springblade.useranalysis.service.UserTagsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import lombok.SneakyThrows;

/**
 * 用户全量标签查询
 */
@Service
@DS("clickhouse")
public class UserTagsServiceImpl extends ServiceImpl<UserTagsMapper, ZUserTagEntity> implements UserTagsService {

	@Autowired
	private UserProfileService userProfileService;
	
	@Autowired
	private DynamicQueryMapper dynamicQueryMapper;

	@SneakyThrows
	@Override
	public List<TagHistory> userTagFlowHisSearch(String userId, String level2Id, String startDate, String endDate) {
		List<TagHistory> rtHis = new ArrayList<TagHistory>();
//		TagColumn weiduColumn = userProfileService.getColumName(tagId, null);
		TagColumn weiduColumn = userProfileService.getColumNameExport(level2Id);
//        SimpleDateFormat format = new SimpleDateFormat("yyyyMM");
//        QueryWrapper<ZUserTagEntity> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("USERID_V", userId)
//                .ge("THEMONTH_V", format.format(start_date))
//                .le("THEMONTH_V", format.format(end_date));
//        queryWrapper.select("THEMONTH_V",weiduColumn.columnE, weiduColumn.columnC, weiduColumn.columnV).orderBy(true,true,"THEMONTH_V");
//        List<Map<String, Object>> maps = baseMapper.selectMaps(queryWrapper);
		List<Map<String, Object>> maps = baseMapper.findOnceTag(userId, startDate,
				endDate, weiduColumn.columnC, weiduColumn.columnE, weiduColumn.columnV);
		Map<Object, List<Map<String, Object>>> groupMap = maps.stream()
				.collect(Collectors.groupingBy(item -> item.get("THEMONTH_V")));

		// 处理返回内容
		if (weiduColumn.columnType.equals("array")) {
			for (Entry<Object, List<Map<String, Object>>> entry : groupMap.entrySet()) {
				String date = entry.getKey().toString();
				Set<String> _tags = new HashSet<String>(), _values = new HashSet<String>();
				List<Map<String, Object>> valueList = entry.getValue();
				for (Map<String, Object> item : valueList) {
					Map<String, Object> tMaps = JSONObject.parseObject(
							JSONObject.toJSONString(item.get(weiduColumn.englishName.toUpperCase())), HashMap.class);
					_tags.addAll((List<String>) tMaps.get("C"));
					_values.addAll((List<String>) tMaps.get("V"));
				}
				TagHistory his = new TagHistory(weiduColumn.cnName, date, _tags, _values);
				rtHis.add(his);
			}
		} else if (weiduColumn.columnType.equals("string")) {
			for (Entry<Object, List<Map<String, Object>>> entry : groupMap.entrySet()) {
				String date = entry.getKey().toString();
				Set<Object> _tags = new HashSet<Object>(), _values = new HashSet<Object>();
				List<Map<String, Object>> valueList = entry.getValue();
				for (Map<String, Object> item : valueList) {
					_tags.add(item.get(weiduColumn.columnC));
					_values.add(item.get(weiduColumn.columnV));
				}
				TagHistory his = new TagHistory(weiduColumn.cnName, date, _tags, _values);
				rtHis.add(his);
			}
		}
		return rtHis;
	}

	@Override
	public List<JSONObject> userTagHisSearch(String userId, String level2Id, String startDate, String endDate) {
//		TagColumn weiduColumn = userProfileService.getColumName(tagId, null);
		TagColumn weiduColumn = userProfileService.getColumNameExport(level2Id);
//		SimpleDateFormat format = new SimpleDateFormat("yyyyMM");
//        QueryWrapper<ZUserTagEntity> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("USERID_V", userId).ge("THEMONTH_V", format.format(start_date))
//                .le("THEMONTH_V", format.format(end_date)).isNotNull(weiduColumn.columnV);
//        queryWrapper.select("THEMONTH_V",weiduColumn.columnE, weiduColumn.columnC, weiduColumn.columnV).orderBy(true,true,"THEMONTH_V");
//        List<Map<String, Object>> maps = baseMapper.selectMaps(queryWrapper);
		List<Map<String, Object>> maps = baseMapper.findOnceTag(userId, startDate,
				endDate, weiduColumn.columnC, weiduColumn.columnE, weiduColumn.columnV);
		List<JSONObject> resultList = new ArrayList<JSONObject>();

		Map<Object, List<Map<String, Object>>> groupMap = maps.stream()
				.collect(Collectors.groupingBy(item -> item.get("THEMONTH_V")));
		// 处理返回内容
		if (weiduColumn.columnType.equals("array")) {
			for (Entry<Object, List<Map<String, Object>>> entry : groupMap.entrySet()) {
				String date = entry.getKey().toString();
				Set<String> valueSet = new HashSet<String>();
				List<Map<String, Object>> valueList = entry.getValue();
				for (Map<String, Object> item : valueList) {
					Map<String, Object> tMaps = JSONObject.parseObject(
							JSONObject.toJSONString(item.get(weiduColumn.englishName.toUpperCase())), HashMap.class);
					List<String> cList = (List<String>) tMaps.get("C");
					valueSet.addAll(cList);
				}
				JSONObject itemObject = new JSONObject();
				itemObject.put("date", date);
				itemObject.put("values", valueSet);
				resultList.add(itemObject);
			}
		} else if (weiduColumn.columnType.equals("string")) {
			for (Entry<Object, List<Map<String, Object>>> entry : groupMap.entrySet()) {
				String date = entry.getKey().toString();
				Set<Object> valueSet = new HashSet<Object>();
				List<Map<String, Object>> valueList = entry.getValue();
				for (Map<String, Object> item : valueList) {
					valueSet.add(item.get(weiduColumn.columnC));
				}
				JSONObject itemObject = new JSONObject();
				itemObject.put("date", date);
				itemObject.put("values", valueSet);
				resultList.add(itemObject);
			}
		}
		return resultList;
	}

	@Override
	public List<UserProfileTagView> userTagSearch(String userId) {
		// 1.查询所有标签
		List<TagColumn> tagColumnList = userProfileService.getAllOnlineTagColumn();
		// 2.组装SQL
		List<String> sqlList = new ArrayList<String>();
		String clickhouseTableName = "ads.Z_USER_TAG_FLAT_ALL";
		String sqlFragment = "select A.*, du.level_1_id, du.level_2_id, du.level_1_name , du.level_2_name, du.english_name from (%s where USERID_V = #{userId}) A left join DIC_USERTAG du on A.tId = du.tag_id";
		for (TagColumn tagColumn : tagColumnList) {
			// 判断Value是哪种数据类型，非字符串需要添加toString()操作
			String sql = "";
			// 字符串格式，形如：SELECT USERID_V,THEMONTH_V,TAG_PROVINCE AS tId,TAG_PROVINCE_C as
			// tCode,TAG_PROVINCE_V as tValue FROM Z_USER_TAG_LAST
			if (tagColumn.columnType.equalsIgnoreCase("string")) {
				String columnV = !"string".equalsIgnoreCase(tagColumn.searchType) ? "toString(" + tagColumn.columnV + ")" : tagColumn.columnV;
				sql = String.format(
						"select distinct USERID_V, THEMONTH_V, %s as tId, %s as tCode, %s as tValue from %s",
						tagColumn.columnE, tagColumn.columnC, columnV, clickhouseTableName);
			} else { // 数组格式，形如：SELECT USERID_V, THEMONTH_V, tId, tCode,tValue FROM Z_USER_TAG_LAST
						// array join EVENT_SPNAME.E as tId,EVENT_SPNAME.C as tCode,EVENT_SPNAME.V as
						// tValue
				String columnV = !"string".equalsIgnoreCase(tagColumn.searchType) ? "toString(tValue)" : "tValue";
				sql = String.format(
						"select distinct USERID_V, THEMONTH_V, tId, tCode, %s from %s array join %s as tId, %s as tCode, %s as tValue",
						columnV, clickhouseTableName, tagColumn.columnE, tagColumn.columnC, tagColumn.columnV);
			}
			sqlList.add(String.format(sqlFragment, sql));
		}
		List<UserProfileTagView> tagViewList = baseMapper.findTagViewByUserId(String.join(" union all ", sqlList),
				userId);
		List<Map<String, Object>> maps = userProfileService.listMaps(
				new QueryWrapper<AdsUserprofileTag>().select("tag_id", "show_his").groupBy("tag_id", "show_his"));
		Map<String, Integer> map = new HashMap<>();
		maps.forEach(v -> {
			map.put((String) v.get("tag_id"), (Integer) v.get("show_his"));
		});
		for (UserProfileTagView tagView : tagViewList) {
			int length = tagView.getTags().size();
			tagView.getTags().removeIf(f -> length > 1 && ObjectUtil.isNull(f.getTagValue()));
			for (UserProfileTagItem tagItem : tagView.getTags()) {
				if (map.containsKey(tagItem.getTagId()) && !map.get(tagItem.getTagId()).equals(0)) {
					tagItem.setShowHis(map.get(tagItem.getTagId()));
				}
			}
		}
		return tagViewList;
	}

	/**
	 * 标签数量校验
	 */
	@Override
	public JSONObject checkTagNum(String tagId, String startDate, String endDate) {
//        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
		TagColumn tagColumn = userProfileService.getColumName(tagId, null);
		String sql = "";
		String whereSql = "";

		// 拼接查询条件
		if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
			whereSql = String.format("and THEMONTH_V >= '%s' and THEMONTH_V <= '%s'", startDate, endDate);
		}
		// 如果列存的是String
		if (tagColumn.columnType.equals("string")) {
			sql = String.format("where %s = '%s' %s", tagColumn.englishName.toUpperCase(), tagId, whereSql);
		} else { // 列存的是Array
			sql = String.format("array join %s as t1 where t1.E = '%s' %s", tagColumn.englishName.toUpperCase(), tagId,
					whereSql);
		}
		List<JSONObject> tagNumList = baseMapper.checkTagNum(sql);
		JSONObject map = new JSONObject();
		map.put("list", tagNumList);
		return map;
	}

	/**
	 * 同二级大类下各标签所占比例
	 */
	@Override
	public JSONObject checkTagRatio(String tagId, String dataDate) {
		JSONObject map = new JSONObject();
		// 获取列名
		TagColumn tagColumn = userProfileService.getColumName(tagId, null);
		String columnName = tagColumn.columnC.toUpperCase();// 要查询的列
		String whereSql = StringUtils.isNotEmpty(dataDate) ? "where THEMONTH_V = '" + dataDate + "'" : "";// 条件
		if (!tagColumn.columnType.equals("string")) {// 列存的是Array
			columnName = "t1.C";
			whereSql = String.format("array join %s as t1 %s ", tagColumn.englishName.toUpperCase(), whereSql);
		}
		List<JSONObject> tagNumList = baseMapper.checkTagRatio(columnName, whereSql);
		if(tagNumList != null && !tagNumList.isEmpty()) {
			//根据leve2Name查询列表
			List<AdsUserprofileTag> tagList = userProfileService.list(Wrappers.<AdsUserprofileTag>lambdaQuery().eq(AdsUserprofileTag::getLevel2Name, tagColumn.cnName));
			Map<String, String> tagName2IdMap = tagList.stream().collect(Collectors.toMap(AdsUserprofileTag::getTagName, AdsUserprofileTag::getTagId));
			//根据tagName反查tagId
			tagNumList.forEach(item -> item.put("tagid", tagName2IdMap.get(item.get("tagname"))));
			//库里面已经不存在该标签值，则不显示
			tagNumList.removeIf(it -> StrUtil.isEmpty(it.getString("tagid")));
		}
		map.put("list", tagNumList);
		return map;
	}

	public List<Map<String, Object>> exportTagList(String fields, QueryWrapper<ZUserTagEntity> TagWrapper,
			String importId,String sqlLimit) {
		if (StringUtils.isBlank(importId)) {
			return baseMapper.exportTagList(fields, TagWrapper);
		}
		return baseMapper.exportTagListBaseImport(fields, TagWrapper, importId,sqlLimit);
	}
	
	public List<Map<String, Object>> exportTagListMonths(String fields,String monthStart,
			String monthEnd,QueryWrapper<ZUserTagEntity> childWrapper,String sqlLimit){
		return baseMapper.exportTagListMonths(fields,monthStart,monthEnd, childWrapper,sqlLimit);
	}

	@Override
	public List<JSONObject> userTagHisDataListByUserId(String userId, String level2Id, String startMonth, String endMonth) {
		TagColumn tagColumn = userProfileService.getColumNameExport(level2Id);
		if(tagColumn == null) {
			return null;
		}
		String sql = "";
		String clickhouseTableName = "ads.Z_USER_TAG_FLAT_ALL";
		if (tagColumn.columnType.equals("string")) {
			String columnV = !"string".equalsIgnoreCase(tagColumn.searchType) ? "toString(" + tagColumn.columnV + ")" : tagColumn.columnV;
			sql = String.format("select distinct THEMONTH_V as the_month, %s as tag_id, %s as tag_name, %s as tag_value from %s where USERID_V = '%s'",
					tagColumn.columnE, tagColumn.columnC, columnV, clickhouseTableName, userId);
		} else { // 列存的是Array
			String columnV = !"string".equalsIgnoreCase(tagColumn.searchType) ? "toString(tmp_tag_value)" : "tmp_tag_value";
			sql = String.format("select distinct THEMONTH_V as the_month, tag_id, tag_name, %s as tag_value from %s array join `%s` as tag_id, `%s` as tag_name, `%s` as tmp_tag_value where USERID_V = '%s'", 
					columnV, clickhouseTableName, tagColumn.columnE, tagColumn.columnC, tagColumn.columnV, userId);
		}
		if(startMonth != null && endMonth != null) {
			sql += String.format(" and THEMONTH_V >= '%s' and THEMONTH_V <= '%s'", startMonth, endMonth);
		}else {
			//获取半年的数据
			LocalDate now = LocalDate.now().minusMonths(1);
			LocalDate halfYear = now.minusMonths(6);
			sql += String.format(" and THEMONTH_V >= '%s' and THEMONTH_V <= '%s'", DateUtil.format(halfYear.atStartOfDay(), "yyyyMM"), DateUtil.format(now.atStartOfDay(), "yyyyMM"));
			
		}
		return dynamicQueryMapper.list(sql);
	}
}
