/**
 * 
 */
package org.springblade.useranalysis.service.normal;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.springblade.useranalysis.entity.clickhouse.AlsRmfResult;
import org.springblade.useranalysis.mapper.clickhouse.AlsRmfResultMapper;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import lombok.Data;

/**
 * @author wookvn
 *
 */
@Service
@DS("clickhouse")
public class AlsRmfResultService extends ServiceImpl<AlsRmfResultMapper, AlsRmfResult> {

	final static int POINT_DIGIT = 2;
	
	/**
	 * 
	 * @return
	 * 
	 * {
	 * 	"indicator": ["R", "M", "F", "L", "G"],
	 *  "dataList": [
	 *  	{
	 *  		value:[],
	 *  		name: "1"  //CLASS值
	 *  	}
	 *  ]
	 * }
	 */
	public JSONObject radarStat(){
		String[] indicator = new String[] {"R", "M", "F", "L", "G"};
		List<JSONObject> dataList = new ArrayList<JSONObject>();
		List<JSONObject> radarList = baseMapper.radarList();
		for(JSONObject jsonObject : radarList) {
			JSONObject item = new JSONObject();
			item.put("name", jsonObject.get("CLASS"));
			Object[] valueList = new Object[indicator.length];
			for(int i = 0; i < indicator.length; i++) {
				valueList[i] = BigDecimal.valueOf(((Double)jsonObject.get(indicator[i]))).setScale(2, BigDecimal.ROUND_HALF_UP);
			}
			item.put("value", valueList);
			dataList.add(item);
		}
		
		JSONObject jsonObject = new JSONObject();
		jsonObject.put("indicator", indicator);
		jsonObject.put("dataList", dataList);
		return jsonObject;
	}
	
	/**
	 * 统计维度 -  F/R可用
	 * @param dimension
	 * @return
	 * {
	 * 	  "xAxis": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], //R/F值
	 * 	  "dataList":[
	 * 		{
	 * 			value:[],
	 *  		name: "1"  // CLASS值
	 * 		}
	 *     ]
	 * }
	 */
	public JSONObject frStat(String dimension) {
		QueryWrapper<AlsRmfResult> queryWrapper = Wrappers.<AlsRmfResult>query()
					.select("CLASS, "+dimension+", uniqCombined(USERID) as userCnt").groupBy("CLASS, " + dimension);
		List<Map<String, Object>> selectMaps = baseMapper.selectMaps(queryWrapper);
		
		JSONObject jsonObject = new JSONObject();
		String[] classArr = new String[] {"1", "2", "3", "4", "5"};
		TreeSet<Long> dimensionSet = selectMaps.stream().map(f -> Long.valueOf(f.get(dimension).toString())).collect(Collectors.toCollection(TreeSet::new));
		
		//分组
		Map<String, List<Map<String, Object>>> resutlMap = selectMaps.stream().collect(Collectors.groupingBy(f -> {
			return f.get(dimension).toString() + "_" + f.get("CLASS").toString();
		}));
		
		List<JSONObject> dataList = new ArrayList<JSONObject>();
		for(String str : classArr) {
			JSONObject item = new JSONObject();
			List<Long> valueList = new ArrayList<Long>();
			item.put("name", str);
			item.put("value", valueList);
			dataList.add(item);
			for(Long _dimension : dimensionSet) {
				String _key = _dimension.toString() + "_" + str;
				List<Map<String, Object>> list = resutlMap.get(_key);
				if(list == null || list.isEmpty()) {
					valueList.add(0L);
				}else {
					valueList.add(Long.valueOf(list.get(0).get("userCnt").toString()));
				}
			}
		}
		jsonObject.put("xAxis", dimensionSet.stream().map(f -> f .toString()+"月").collect(Collectors.toList()));
		jsonObject.put("dataList", dataList);
		return jsonObject;
	}

	/**
	 * 
	 * @return
	 * {
	 * 	  "xAxis": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12], //R/F值
	 * 	  "dataList":[
	 * 		{
	 * 			value:[],
	 *  		name: "1"  // CLASS值
	 * 		}
	 *     ]
	 * }
	 */
	public JSONObject mlgStat(String dimension) {
		List<String> countSql = new ArrayList<String>();
		List<Range> rangeList = rangeMap.get(dimension);
		for(int i = 0; i < rangeList.size(); i++) {
			Range range = rangeList.get(i);
			if(range.getEnd() == 0) {
				countSql.add(String.format("uniqCombinedIf(USERID, %s >= %s) as a%s", dimension, range.getStart(), i));
			}else {
				countSql.add(String.format("uniqCombinedIf(USERID, %s >= %s and %s < %s) as a%s", dimension, range.getStart(), dimension, range.getEnd(), i));
			}
		}
		List<JSONObject> resultList = baseMapper.mlgList(String.join(",", countSql), dimension);
		JSONObject result = new JSONObject();
		JSONArray list = new JSONArray();
		for (JSONObject object : resultList) {
			JSONObject data = new JSONObject();
			data.put("name", object.get("CLASS"));
			List<Object> valueList = new ArrayList<>();
			for(int i = 0; i < rangeList.size(); i++) {
				valueList.add(object.get("a" + i));
			}
			data.put("value", valueList);
			list.add(data);
		}
		
		result.put("dataList", list);
		result.put("xAxis", rangeList.stream().map(f -> f.getLegend()).collect(Collectors.toList()));
		return result;
	}
	
	
	static Map<String, List<Range>> rangeMap = new HashMap<String, List<Range>>();
	
	static {
		List<Range> mRange = new ArrayList<Range>();
		List<Range> lRange = new ArrayList<Range>();
		List<Range> gRange = new ArrayList<Range>();
		//M区间
		mRange.add(new Range(0, 10));
		mRange.add(new Range(10, 30));
		mRange.add(new Range(30, 50));
		mRange.add(new Range(50, 70));
		mRange.add(new Range(70, 90));
		mRange.add(new Range(90, 110));
		mRange.add(new Range(110, 130));
		mRange.add(new Range(130, 0));
		//L区间
		lRange.add(new Range(0, 5, "G"));
		lRange.add(new Range(5, 10, "G"));
		lRange.add(new Range(10, 15, "G"));
		lRange.add(new Range(15, 20, "G"));
		lRange.add(new Range(20, 0, "G"));
		//G区间
		gRange.add(new Range(0, 10, "元"));
		gRange.add(new Range(10, 20, "元"));
		gRange.add(new Range(20, 30, "元"));
		gRange.add(new Range(30, 40, "元"));
		gRange.add(new Range(40, 50, "元"));
		gRange.add(new Range(50, 60, "元"));
		gRange.add(new Range(60, 70, "元"));
		gRange.add(new Range(70, 80, "元"));
		gRange.add(new Range(80, 90, "元"));
		gRange.add(new Range(90, 0, "元"));
		
		rangeMap.put("M", mRange);
		rangeMap.put("l", lRange);
		rangeMap.put("G", gRange);
	}
	
	@Data
	static class Range{
		int start;
		int end;
		String unit;
		String legend;
		
		public Range(int start, int end) {
			this.start = start;
			this.end = end;
			this.legend = end == 0 ? start+"以上" : start+"-"+end;
		}
		
		public Range(int start, int end, String unit) {
			this.start = start;
			this.end = end;
			this.legend = end == 0 ? start+unit+"以上" : start+"-"+end+unit;
		}
	}
}
