package org.springblade.useranalysis.service.normal;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

import org.springblade.useranalysis.entity.MidCpidUserId;
import org.springblade.useranalysis.mapper.MidCpidUserIdMapper;
import org.springframework.stereotype.Service;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import cn.hutool.core.date.DateUtil;

/**
 * @author Administrator
 *
 */
@Service
@DS("slave")
public class MidCpidUserIdService extends ServiceImpl<MidCpidUserIdMapper, MidCpidUserId> {
	public static final int PAGE_SIZE = 3000;
	final String monthPattern = "yyyyMM";
	final String dayPattern = "yyyyMMdd";

	public void clear() {
		baseMapper.clear();
	}

	public void findAll(String cpid, int thedayStart, int thedayEnd) {
		baseMapper.findAll(cpid, thedayStart, thedayEnd);
	}

	public void updateOld(int themonth) {
		baseMapper.updateOld(themonth);
	}
	
	public int getNewCount() {
		return baseMapper.getNewCount();
	}
	
	public int getOldCount() {
		return baseMapper.getOldCount();
	}

	public List<MidCpidUserId> getPhoneList(int type, int page) {
		LambdaQueryWrapper<MidCpidUserId> queryWrapper = Wrappers
				.<MidCpidUserId>query().lambda()
				.orderByAsc(MidCpidUserId::getUserId)
				.eq(MidCpidUserId::getIsOldUser, type);
		IPage<MidCpidUserId> userPage = new Page<>(page, PAGE_SIZE);
		IPage<MidCpidUserId> iPage = baseMapper.selectPage(userPage, queryWrapper);
		List<MidCpidUserId> list = iPage.getRecords();
		return list;
	}

	/**
	 * 
	 * @param cpid
	 * @param date 例如：202106
	 * @return
	 */
	public void handle(String cpid, String dateStr) {
		Date date = DateUtil.parse(dateStr, monthPattern);
		Instant instant = date.toInstant();
		ZoneId zoneId = ZoneId.systemDefault();
		LocalDateTime dateTime = instant.atZone(zoneId).toLocalDateTime();
		LocalDateTime lastDay = dateTime.plusMonths(1).minusDays(1);
		int thedayStart = Integer.parseInt(DateTimeFormatter.ofPattern(dayPattern).format(dateTime));
		int thedayEnd = Integer.parseInt(DateTimeFormatter.ofPattern(dayPattern).format(lastDay));
		int themonth = Integer.parseInt(DateTimeFormatter.ofPattern(monthPattern).format(dateTime));
		clear();
		findAll(cpid, thedayStart, thedayEnd);
		updateOld(themonth);
	}
}
