/**
 * 
 */
package org.springblade.useranalysis.service.normal;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.stream.Collectors;

import org.springblade.useranalysis.entity.AdsUserprofileTag;
import org.springblade.useranalysis.entity.TagColumn;
import org.springblade.useranalysis.mapper.clickhouse.NewUserTagsMapper;
import org.springblade.useranalysis.service.UserProfileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 * 
 *         新版本的大宽表查询
 */
@Service
@DS("clickhouse")
public class NewUserTagsService {

	@Autowired
	private NewUserTagsMapper newUserTagsMapper;
	
	@Autowired
	private UserProfileService userProfileService;
	
	@Autowired
	private ProductInfoService productInfoService;
	

	/**
	 * 查询
	 * @param userId
	 * @return
	 */
	public JSONObject getProductListByUserId(String userId) {
		JSONObject result = new JSONObject();
		//1.根据用户ID查询近半年的订购产品{产品ID}  条件：快照日期在今年，并且退订时间也在今年
		List<String> spidList = newUserTagsMapper.getProductValList(userId);
		if(spidList == null || spidList.isEmpty()) {
			return result;
		}
		// 查询产品标签字典列表 - 主要为了组装SQL
		List<JSONObject> tagList = userProfileService.getProductTagList();
		//去除掉未订购标签/关注公众号/绑定小程序
		tagList.removeIf(item -> 
				"event_unspname".equalsIgnoreCase(item.getString("english_name"))
				|| "tag_wxpublic_follow".equalsIgnoreCase(item.getString("english_name"))
				|| "tag_wxapp_bind".equalsIgnoreCase(item.getString("english_name")));
		List<String> columnList = new ArrayList<>();
		columnList.add("THEMONTH_V");
		for (JSONObject jsonObject : tagList) {
			String ctype = jsonObject.getString("tag_c_type");
			String column = jsonObject.getString("english_name").toUpperCase();
			jsonObject.put("columnName", column);
			String columnE = column, columnC = column + "_C", columnV = column + "_V";
			if (!ctype.equalsIgnoreCase("string")) {
				columnE = String.format("arrayFirst(t -> t != '', `%s`) as %s", column + ".E", columnE);
				columnC = String.format("arrayFirst(t -> t != '', `%s`) as %s", column + ".C", columnC);
				columnV = String.format("toString(arrayFirst(t -> toString(t) != '', `%s`)) as %s", column + ".V", columnV);
			}else {
				columnV = "toString(" + columnV + ") as " + columnV;
			}
			columnList.add(columnE);
			columnList.add(columnC);
			columnList.add(columnV);
		}
		
		//tagValList -> SpidList [根据SPID查询订购产品列表]
		List<String> validSpidList = spidList.stream().filter(f -> ObjectUtil.isNotNull(f)).collect(Collectors.toList());
		List<JSONObject> spList = productInfoService.spList(validSpidList);
		final Map<String, String> spMap = new HashMap<>();
		spList.forEach(item -> spMap.put(item.getString("SPID"), item.getString("SPNAME")));
		
		//tagValList - 根据tagValue查询用户的产品标签信息
		//问：为什么根据tagValue(SPID)查询产品标签列表，而不是根据标签ID(tagId)查询？？
		//答：联通要求显示全部订购的产品。但是，订购产品名不在字典标签表都归类为其他，只有tagValue(SPID)是唯一的。
		List<JSONObject> dataList = newUserTagsMapper.getProductDataList(String.join(",", columnList), userId, validSpidList);
		dataList.forEach(item -> {
			//非常不友好的操作<满足联通要求> - 退订时间1970-01-01 显示 未退订
			if("1970-01-01".equalsIgnoreCase(item.getString("EVENT_CANCEL_DATE_V"))) {
				item.put("EVENT_CANCEL_DATE_V", "未退订");
			}
			//产品名称_C替换[因为：有部分产品名称为其他，联通要求全部显示真实名称；EVENT_SPNAME_V存储的是SPID，根据SPID反查订购产品显示]
			String spid = item.getString("EVENT_SPNAME_V");
			if(spMap.containsKey(spid)) {
				item.put("EVENT_SPNAME_C", spMap.get(spid));
			}
		});
		dataList.sort((t, t1) -> t1.getString("EVENT_ORDER_DATE_V").compareTo(t.getString("EVENT_ORDER_DATE_V")));
		result.put("dataList", dataList);
		result.put("tagList", tagList);
		return result;
	}

	/**
	 * 根据用户ID查询最新的记录
	 * 
	 * @param userId
	 * @return
	 */
	public List<JSONObject> getXxhAlgDataListByUserId(String userId) {
		//查询大网和算法标签数据
		AdsUserprofileTag entity = new AdsUserprofileTag();
		entity.setIsOnline("1");
		entity.setLevel1Id("A");
		List<TagColumn> tagColumnList = userProfileService.getTagColumnList(entity);
		entity.setLevel1Id("C");
		tagColumnList.addAll(userProfileService.getTagColumnList(entity));
		//大网标签不显示：月使用小时数和月使用天数和APP偏好
		tagColumnList.removeIf(f -> StrUtil.containsAnyIgnoreCase(f.columnE, new String[] {"event_app_use_hours", "event_app_use_days"}));
		// 组装SQL
		List<String> sqlList = new ArrayList<String>();
		String clickhouseTableName = "ads.Z_USER_TAG_FLAT_ALL";
		StringBuilder sqlFragmentBuffer = new StringBuilder();
		sqlFragmentBuffer.append(
				"\nselect A.*, du.level_1_id, du.level_2_id, du.level_1_name , du.level_2_name, du.english_name, du.show_his from")
				.append("\n(").append("\n%s")
				.append("\nselect distinct THEMONTH_V as theMonth, tId, tCode, tValue from " + clickhouseTableName)
				.append("\nwhere USERID_V = #{userId} and tId != '' and tCode != '' and tValue != '' order by THEMONTH_V desc limit 1")
				.append("\n)").append("A left join DIC_USERTAG du on A.tId = du.tag_id\n");
		for (TagColumn tagColumn : tagColumnList) {
			// 字符串格式
			if (tagColumn.columnType.equalsIgnoreCase("string")) {
				String withSql = String.format("with %s as tId, %s as tCode, toString(%s) as tValue", tagColumn.columnE,
						tagColumn.columnC, tagColumn.columnV);
				sqlList.add(String.format(sqlFragmentBuffer.toString(), withSql));
			} else { // 数组格式
				//数组获取前3条记录
				String withSql = String.format(
						"with arrayFirst(t -> t != '', `%s`) as tId, toString(arrayFirst(t -> toString(t) != '', `%s`)) as tValue, arrayStringConcat(arraySlice(`%s`, 1, 3), ',') as tCode",
						tagColumn.columnE, tagColumn.columnV, tagColumn.columnC);
				if(StrUtil.containsAnyIgnoreCase(tagColumn.columnE, "tag_app_pref")) {
					withSql = String.format(
							"with arrayFirst(t -> t != '', `%s`) as tId, toString(arrayFirst(t -> toString(t) != '', `%s`)) as tValue, arrayFirst(t -> t != '', `%s`) as tCode",
							tagColumn.columnE, tagColumn.columnV, tagColumn.columnC);
				}
				sqlList.add(String.format(sqlFragmentBuffer.toString(), withSql));
			}
		}
		String sql = String.join("\nunion all\n", sqlList);
		return newUserTagsMapper.getTagViewByUserId(sql, userId);
	}

	/**
	 * 近半年产品分布图
	 * @param userId
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	public List<JSONObject> userTagProductHisSearch(String userId, String startDate, String endDate){
		List<JSONObject> resultList = new ArrayList<JSONObject>();
		Date _startDate = DateUtil.parse(startDate, "yyyy-MM"), _endDate = DateUtil.parse(endDate, "yyyy-MM");
		//查询用户的产品标签数据
		QueryWrapper<?> queryWrapper = Wrappers.query();
		queryWrapper.eq("USERID_V", userId).isNotNull("EVENT_SPNAME_V").ne("EVENT_SPNAME_V", "''")
			.ge("THEMONTH_V", DateUtil.format(_startDate, "yyyyMM"))
			.ge("EVENT_CANCEL_DATE_V", DateUtil.format(_startDate, "yyyy-MM-dd"));
		//查询快照日期在今年，并且退订时间也在今年的产品列表{退订时间不在日期范围内，页面上图表无法展示，数据无效}
		List<JSONObject> productDateList = newUserTagsMapper.getProductDateList(queryWrapper, DateUtil.format(_endDate, "yyyy-MM-dd"));
		//获取该用户SPID对应的产品列表
		Set<String> spidList = productDateList.stream().filter(f -> ObjectUtil.isNotNull(f.get("EVENT_SPNAME_V"))).map(item -> item.get("EVENT_SPNAME_V").toString()).distinct().collect(Collectors.toSet());
		if(spidList.isEmpty()) {
			return resultList;
		}
		List<JSONObject> spList = productInfoService.spList(new ArrayList<>(spidList));
		final Map<String, String> spMap = new HashMap<>();
		spList.forEach(item -> spMap.put(item.getString("SPID"), item.getString("SPNAME")));
		
		/**  EVENT_ORDER_DATE_V|EVENT_CANCEL_DATE_V|EVENT_SPNAME_V|
			------------------+-------------------+--------------+
			        2022-01-30|         2022-08-01|1218          |
			        2022-01-30|         2022-07-03|1218          |
			        2022-04-24|         2022-08-01|261           |
			        2022-04-15|         2022-08-01|269           |
			------------------+-------------------+--------------+  
			需求：显示订购产品的趋势柱状图，用户画像->用户标签->用户全息档案<近半年产品订购情况>。
			处理方式：
			1.首先根据EVENT_SPNAME_V分组，计算出来每个时间段内的日期字符串数组；
			2.然后组内日期数组做交集：2种情况,(1)无结果，合并2个数组为结果数组；(2)有结果，作为结果数组；
			3.最终得到每个EVENT_SPNAME_V的结果数组，组装数据。
		 */
		Map<Object, List<JSONObject>> groupMap = productDateList.stream().collect(Collectors.groupingBy(item -> item.get("EVENT_SPNAME_V")));
		Map<String, List<String>> productDateMap = new HashMap<>();
		for (Entry<Object, List<JSONObject>> entry : groupMap.entrySet()) {
			String spId = entry.getKey().toString();
			String spName = spMap.get(spId);
			if(spName == null) {
				continue;
			}
			productDateMap.put(spName, new ArrayList<>());
			List<JSONObject> dataList = entry.getValue();
			for(JSONObject jsonObject : dataList) {
				List<String> calcDateArray = calcDateArray(jsonObject.getString("EVENT_ORDER_DATE_V"), jsonObject.getString("EVENT_CANCEL_DATE_V"));
				List<String> _tempList = productDateMap.get(spName).stream().filter(f -> calcDateArray.contains(f)).collect(Collectors.toList());
				if(_tempList.isEmpty()) {
					productDateMap.get(spName).addAll(calcDateArray);
				}else {
					productDateMap.put(spName, _tempList);
				}
			}
		}
		//对每个产品计算出来的数组，组装数据
//		DateTime _startDate = DateUtil.parse(startDate, "yyyy-MM-dd");
//		DateTime _endDate = DateUtil.parse(endDate, "yyyy-MM-dd");
		//循环日期处理
		DateUtil.rangeConsume(_startDate, _endDate, DateField.MONTH, t -> {
			String curDateStr = DateUtil.format(t, "yyyyMM");
			Set<Object> valueSet = new HashSet<Object>();
			for (Entry<String, List<String>> entry : productDateMap.entrySet()) {
				if(entry.getValue().contains(curDateStr)) {
					valueSet.add(entry.getKey());
				}
			}
//			if(!valueSet.isEmpty()) {
			JSONObject itemObject = new JSONObject();
			itemObject.put("date", curDateStr);
			itemObject.put("values", valueSet);
			resultList.add(itemObject);
//			}
		});
		return resultList;
	}
	
	private static List<String> calcDateArray(String startDateStr, String endDateStr) {
		DateTime startDate = DateUtil.parse(startDateStr, "yyyy-MM-dd").setField(DateField.DAY_OF_MONTH, 1);
		DateTime endDate = DateUtil.parse(endDateStr, "yyyy-MM-dd").setField(DateField.DAY_OF_MONTH, 1);
		return DateUtil.rangeFunc(startDate, endDate, DateField.MONTH, t -> DateUtil.format(t, "yyyyMM"));
	}
}