/**
 * 
 */
package org.springblade.useranalysis.service.normal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springblade.useranalysis.dto.UserGroupAnalysisRequest;
import org.springblade.useranalysis.dto.UserGroupAnalysisResult;
import org.springblade.useranalysis.entity.AdsUserprofileTag;
import org.springblade.useranalysis.mapper.clickhouse.UserGroupSumDataMapper;
import org.springblade.useranalysis.service.UserProfileService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.util.StrUtil;

/**
 * @author wookvn
 *
 */
@Service
@DS("clickhouse")
public class UserGroupSumDataService {

	@Autowired
	private UserGroupSumDataMapper userGroupSumDataMapper;
	
	@Autowired
	private UserProfileService userProfileService;
	
	private static List<JSONObject> columnListCache = null;
	
	/**
	 * 删除groupId
	 * @param groupId
	 */
	public void deleteByGroupId(String groupId) {
		userGroupSumDataMapper.deleteByGroupId(groupId);
	}
	
	/**
	 * @param groupId
	 * @param month
	 * @return
	 */
	public long count(String groupId, String month) {
		return userGroupSumDataMapper.count(groupId, month);
	}
	
	/**
	 * 增删字段列
	 * @param sql
	 */
	public void addOrDelColumn(String sql) {
		userGroupSumDataMapper.executeSql(sql);
	}
	
	/**
	 * 批量插入数据
	 * @param sql
	 */
	public void batchSaveData(String sql) {
		userGroupSumDataMapper.executeSql(sql);
	}
	
	/**
	 * 获取字段列集合
	 * @return
	 */
	public List<JSONObject> getColumnList(){
		return userGroupSumDataMapper.selectList("DESCRIBE TABLE ads.BIZ_USER_GROUP_TAG_SUM");
	}
	
	/**
	 * 统计分析
	 * @param groupId
	 * @param level2Id
	 * @return
	 */
	public List<UserGroupAnalysisResult> analyis(UserGroupAnalysisRequest userGroupAnalysisRequest){
		//1.根据level2Id查询标签字典表数据
		LambdaQueryWrapper<AdsUserprofileTag> queryWrapper = Wrappers.<AdsUserprofileTag>lambdaQuery().eq(AdsUserprofileTag::getLevel2Id, userGroupAnalysisRequest.getLevel2Id());
		List<AdsUserprofileTag> tagList = userProfileService.list(queryWrapper);
		Map<String, AdsUserprofileTag> tagMap = tagList.stream().collect(Collectors.toMap(f -> f.getEnglishName()+"_"+f.getTagId(), t -> t));
		//2.查询汇总表列
		List<JSONObject> columnList = getColumnListFromCache();
		List<String> selectColumnList = new ArrayList<>();
		for(JSONObject jsonObject : columnList) {
			String columnName = jsonObject.getString("name");
			if(tagMap.containsKey(columnName)) {
				selectColumnList.add(columnName);
			}
		}
		selectColumnList.add("the_month");
		//3.查询汇总表
		StringBuilder selectSql = new StringBuilder("select ")
						.append(String.join(",", selectColumnList))
						.append(" from ads.BIZ_USER_GROUP_TAG_SUM")
						.append(" where group_id = '"+userGroupAnalysisRequest.getGroupId()+"'");
		if(StrUtil.isNotBlank(userGroupAnalysisRequest.getStartDate())) {
			selectSql.append(" and the_month >= '"+userGroupAnalysisRequest.getStartDate()+"'");
		}
		if(StrUtil.isNotBlank(userGroupAnalysisRequest.getEndDate())) {
			selectSql.append(" and the_month <= '"+userGroupAnalysisRequest.getEndDate()+"'");
		}
//		log.info(selectSql.toString());
		List<JSONObject> dataList = userGroupSumDataMapper.selectList(selectSql.toString());
		List<UserGroupAnalysisResult> resultList = new ArrayList<>();
		for(JSONObject jsonObject : dataList) {
			String month = jsonObject.getString("the_month");
			for(String key : selectColumnList){
				Long value = jsonObject.getLong(key);
				AdsUserprofileTag userprofileTag = tagMap.get(key);
				//过滤>0的值
				if(userprofileTag != null && value != null && value > 0) {
					resultList.add(new UserGroupAnalysisResult(userprofileTag.getLevel2Id(), userprofileTag.getLevel2Name(),
							userprofileTag.getTagId(), userprofileTag.getTagName(), month, value, userprofileTag.getTagOrder()));
				}
			}
		}
		//按照标签的顺序排序
		Collections.sort(resultList, (t, t1) -> t.getTagOrder() - t1.getTagOrder());
		return resultList;
	}
	
	/**
	 * 从缓存中获取字段列表
	 * @return
	 */
	public List<JSONObject> getColumnListFromCache(){
		if(columnListCache == null) {
			UserGroupSumDataService.columnListCache = getColumnList();
		}
		return UserGroupSumDataService.columnListCache;
	}
	
	/**
	 * 清除缓存
	 */
	public void cleanColumnListCache() {
		UserGroupSumDataService.columnListCache = null;
	}
	
}