package org.springblade.useranalysis.task;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import org.apache.commons.codec.digest.DigestUtils;
import org.springblade.useranalysis.common.utils.EnDecHelper;
import org.springblade.useranalysis.entity.UserGroupEntity;
import org.springblade.useranalysis.enums.UserGroupUseStatus;
import org.springblade.useranalysis.service.UserGroupService;
import org.springblade.useranalysis.service.normal.UserGroupUserIdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.http.HttpUtil;
import lombok.extern.slf4j.Slf4j;

@Component
@Configuration // 1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling // 2.开启定时任务
@Slf4j
public class PushUserIdTask {

	@Autowired
	private UserGroupService userGroupService;

	@Autowired
	private UserGroupUserIdService userGroupUserIdService;

	@Value("${smartOrder.pushUseridUrl}")
	private String pushUseridUrl;

	@Value("${smartOrder.md5pwd}")
	private String md5pwd;

	private static boolean run = false;

	/**
	 * 智能订购页推送用户Id
	 */
	@Scheduled(cron = "0 0/1 2-4 * * ?")
//	@Scheduled(cron = "0/5 * * * * ?")
	private void pushUserId() {
		try {
			if (run) {
				return;
			}
			run = true;
			// 查询状态为处理中的人群
			List<UserGroupEntity> userGroupEntities = userGroupService.list(new LambdaQueryWrapper<UserGroupEntity>()
					.eq(UserGroupEntity::getUseStatus, UserGroupUseStatus.processing));

			// 根据groupId查询clickhouse中ads.BIZ_USER_GROUP_USERID中的user_id
			Integer limit = 50000;
			userGroupEntities.parallelStream().forEach(x -> {
				try {
					String userId = "0";
					while (true) {
						List<String> resultList = userGroupUserIdService.selectByGroupId(x.getGroupId(), userId, limit);
						if (resultList == null || resultList.isEmpty()) {
							break;
						}
						// 处理数据
						pushData(x.getGroupId(), resultList);
						userId = resultList.get(resultList.size() - 1);
					}
					x.setUseStatus(UserGroupUseStatus.success);
					userGroupService.updateById(x);

				} catch (Exception e) {
					log.error(ExceptionUtil.stacktraceToString(e));
					log.info("【推送userid】推送userid失败｜groupid={}", x.getGroupId());
					x.setUseStatus(UserGroupUseStatus.failure);
					userGroupService.updateById(x);
				}
			});
		} catch (Exception e) {
			log.error(ExceptionUtil.stacktraceToString(e));
		} finally {
			run = false;
		}

	}

	/**
	 * 处理流程
	 * @param groupId
	 * @param resultList
	 * @throws Exception
	 */
	public void pushData(String groupId, List<String> resultList) throws Exception {
		if (resultList != null && resultList.size() > 0) {
			String userIds = resultList.parallelStream().map(EnDecHelper::encode).filter(Objects::nonNull)
					.collect(Collectors.joining(","));
			Map<String, Object> paraMap = new HashMap();
			long timestamp = System.currentTimeMillis() / 1000;
			paraMap.put("timestamp", timestamp + "");
			paraMap.put("signature", DigestUtils.md5Hex(timestamp + md5pwd));
			paraMap.put("userId", userIds);
//			String params = GsonUtils.toJson(paraMap);
//			String results = HttpUtils.doPostByJson(pushUseridUrl + groupId, null, params);
			String params = JSON.toJSONString(paraMap);
			String results = HttpUtil.post(pushUseridUrl + groupId, params);
			log.info("【同步结果】同步结果｜results={}", results);
			JSONObject jsonObject = JSONObject.parseObject(results);
			if (jsonObject != null) {
				String code = jsonObject.getString("code");
				if (!"10000".equals(code)) {
					throw new RuntimeException("同步失败!");
				}
			}

		}

	}
}
